from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.hierarchies.portal_find_sponsor import PortalFindSponsor


class PortalCreateSponsor(StandardStep):
    Name = "portal_create_sponsor.py"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def _pre_processor(self):
        PortalFindSponsor(self.service, self.scenario).run()

    def ignore(self):
        return self.service.context["sponsor_id"] is not None

    def sponsor_name(self): return self.scenario.sponsor()

    def data(self):
        return {"name": self.sponsor_name(), "description": "", "active": True}

    def _execute(self):
        super()._execute()
        self.service.hierarchies_new_sponsor(data=self.data())



from eclinical.scaffold.inputs.component.radio import Radio


class Application:
    @classmethod
    def input(cls, msg, select=("portaladmin", "portal", "ctms", "etmf", "design", "edc", "iwrs", "eConsent(暂时不支持)", "pv",
                     "IDP(暂不支持)", "非Application")):
        radio = Radio(msg, select)
        try:
            return radio.get(int(input(f"{radio}\n")))
        except Exception as e:
            return cls.input(f"输入非法，请重新输入{msg}")



import cjen
from cjen import MetaJson


class Sponsor(MetaJson):
    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.name")
    def name(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.active")
    def active(self): ...


class QuerySponsors(MetaJson):
    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=Sponsor)
    @cjen.operate.json.one(json_path="$.payload.sponsorExtDtoList")
    def sponsorExtDtoList(self) -> list[Sponsor]: ...





import cjen

from cjen import MetaJson


class Env(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.name")
    def name(self): ...


class Envs(MetaJson):

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=Env)
    @cjen.operate.json.one(json_path="$.payload")
    def payload(self) -> list[Env]: ...



import cjen
from cjen import MetaJson


class StudySite(MetaJson):
    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.siteName")
    def siteName(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.siteCode")
    def siteCode(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.studyId")
    def studyId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...


class SitesOfStudy(MetaJson):
    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.payload.nextPage")
    def nextPage(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=StudySite)
    @cjen.operate.json.one(json_path="$.payload.list")
    def list(self) -> list[StudySite]: ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...



from cjen.sco.scenario import Scenario


class PortalScenario(Scenario):
    def __init__(self, scenario_dir):
        super().__init__(scenario_dir)

    def sponsor(self):
        return self.get("sponsor").get("name")

    def study(self):
        return self.get("study").get("name")

    def life_cycle(self):
        return self.get("life_cycle").get("env")

    def user_group(self):
        return self.get("user_groups").get("name")

    def user_group_sites(self):
        return self.get("user_groups").get("sites")

    def user(self):
        return self.get("user").get("name")

    def role(self):
        return self.get("user_role").get("name")


from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario


class PortalFindUser(StandardStep):
    Name = "portal_find_user.py"
    Id = "user_id"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def user_name(self):
        return self.scenario.user().upper()

    def call_back(self, **kwargs):
        self.service.context[self.Id] = None
        for user in kwargs.get("users").list():
            if user.loginName() == self.user_name():
                self.service.context[self.Id] = user.id()

    def _execute(self):
        super()._execute()
        self.service.user_api_user_query(path_variable=dict(pageNo=1))



class Meta:

    def __init__(self, name, props=[]):
        self.props = props
        self.name = name

    @property
    def clazz_name(self):
        return "".join([elem.capitalize() for elem in self.name.split("_")])

    def json(self):
        content = "import cjen\nfrom cjen import MetaJson\n\n\n" + f"class {self.clazz_name}(MetaJson):\n"
        content += "    \"\"\"\n    相关使用详情可查询:\n" \
                   "    装饰器: https://github.com/thcpc/cjen#42-metajson-%E8%A3%85%E9%A5%B0%E5%99%A8" \
                   "\n" + \
                   "    \"\"\"\n\n"
        for prop in self.props:
            content += f"    @cjen.operate.common.value\n"
            content += f"    @cjen.operate.json.one(json_path=\"请填写jsonpath\")\n"
            content += f"    def {prop}(self): ...\n\n"
        return content

    def mysql(self):
        content = "import cjen\nfrom cjen import MetaMysql\n\n\n" + f"class {self.clazz_name}(MetaMysql):\n"
        content += "    \"\"\"\n    相关使用详情可查询:\n" \
               "    装饰器: https://github.com/thcpc/cjen#43-metamysql-%E8%A3%85%E9%A5%B0%E5%99%A8\n" +\
               "    \"\"\"\n\n"
        for prop in self.props:
            content += f"    @cjen.operate.common.value\n"
            content += f"    def {prop}(self): ...\n\n"
        return content



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.ctms_scenario import CtmsScenario
from eclinical.standard.steps.ctms.ctms_find_study import CtmsFindStudy


class CtmsSubmitStudyBasicInfo(StandardStep):
    Name = "ctms_submit_study_basic_info.py"

    def __init__(self, service, scenario: CtmsScenario):
        super().__init__(service, scenario)

    def _pre_processor(self):
        CtmsFindStudy(self.service, self.scenario).run()

    def is_randomized(self):
        return "IWRS" in self.scenario.apps()

    def data(self):
        return {
            "blindMethods": 0,
            "craReview": False,
            "daysToResolveQuery": 100,
            "endDate": 2519827200000,
            "id": self.service.context[CtmsFindStudy.Info].get("id"),
            "intervalDays": 10000,
            "isDraft": False,
            "mainPurpose": 1,
            "mandatoryForAttachments": 0,
            "name": self.service.context[CtmsFindStudy.Info].get("name"),
            "numOfSite": 3,
            "numOfSubject": 100,
            "randomize": self.is_randomized(),
            "saftyReview": False,
            "startDate": 1655971679755,
            "studyPhase": 1,
            "studyType": 0
        }

    def _execute(self):
        super()._execute()
        self.service.study_management_submit_info(data=self.data())



from eclinical.scaffold.inputs.component.radio import Radio


class LifeCycle:
    @classmethod
    def input(cls, msg: str = "请输入项目阶段", select=("dev", "uat", "prod")):
        radio = Radio(msg, select)
        try:
            return radio.get(int(input(f"{radio}\n")))
        except Exception as e:
            return cls.input("输入非法，请重新输入项目阶段")


import os

from eclinical.environment.environment import Environment


class SmokeEnvironment(Environment):
    def __init__(self, *, envir, path):
        super().__init__(envir, os.path.join(path, "smoke_environment.yaml"))



from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario


class PortalFindUserRole(StandardStep):
    Name = "portal_role_list.py"
    Id = "user_role_id"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def role(self):
        return self.scenario.role()

    def _execute(self):
        super()._execute()
        self.service.user_api_role_list()

    def call_back(self, **kwargs):
        for r in kwargs.get("role_list").payload():
            if r.code() == self.role():
                self.service.context[PortalFindUserRole.Id] = r.id()





import os.path

from eclinical import Environment
from eclinical.standard.portal.portal_api import PortalApi

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.scenarios.sco_init import ScoInit
from eclinical.standard.steps.portal.user_groups.portal_create_usergroup import PortalCreateUserGroup
from eclinical.standard.steps.portal.user_groups.portal_get_lifecycle_hierarchies import PortalGetLifeCycleHierarchies
from eclinical.standard.steps.portal.user_groups.portal_get_user_group_info import PortalGetUserGroupInfo

#  创建UserGroup, 并且分配User, Sponsor, Study, Site.
from eclinical.standard.steps.portal.user_groups.portal_set_lifecycle_hierarchies import PortalSetLifeCycleHierarchies


class ScoUserGroup:
    def __init__(self, scenario_dir, environment: Environment):
        self.scenario = PortalScenario(scenario_dir)
        self.service = PortalApi(environment)

        # 创建用户组
        self.scenario.append_step(PortalCreateUserGroup, self.service)
        # 获取未添加到User Group中的 User
        self.scenario.append_step(PortalGetUserGroupInfo, self.service)
        # 选择lifecycle, 获取对应lifecycle的sponsor, study, site 信息
        self.scenario.append_step(PortalGetLifeCycleHierarchies, self.service)
        # 分配 Sponsor, Study, Site
        self.scenario.append_step(PortalSetLifeCycleHierarchies, self.service)

    def run(self): self.scenario.run()


if __name__ == '__main__':
    sco_dir = os.path.join("new_study")
    sco = ScoUserGroup(sco_dir, Environment.loader("DEV_03"))
    sco.run()
    # a = "a"
    # print(f'{{a}}')



from eclinical.scaffold.inputs.component.text import Text


class MetaAttrs:
    @classmethod
    def input(cls, msg="请输入对象属性，多个属性以','分割", default=None, default_key=None):
        text = Text(msg, default, default_key)
        attrs = text.get(input(f"{text}\n")).split(",")
        if len(attrs) != len(set(attrs)):
            return cls.input("输入非法，meta的类型属性不能有相同的名字")
        return attrs


if __name__== '__main__':
    a  = [1,2,1]
    b = set(a)
    print(len(a),len(b))


from eclinical.scaffold.inputs.component.text import Text


class Port:

    @classmethod
    def input(cls, msg: str = "", default=None, default_key=None):
        text = Text(msg, default, default_key)
        try:
            return int(text.get(input(f"{text}\n")))
        except Exception as e:
            return cls.input("输入非法，请重新输入", default, default_key)



import cjen

from eclinical.environment.environment import Environment
from eclinical.service._login_service import _LoginService


class PortalLoginService(_LoginService):
    @cjen.context.add(content=dict(system="ADMIN"))
    def __init__(self, environment: Environment = None):
        super(PortalLoginService, self).__init__(environment)
        self.auth(data=self.user_pwd())
        self.entry_portal()

    @cjen.http.post_mapping(uri="admin/user/entry/portal")
    def entry_portal(self, resp=None, **kwargs): ...



import cjen
from cjen import MetaJson


class RelUser(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.userGroupName")
    def userGroupName(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.userGroupUserRelId")
    def userGroupUserRelId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.userId")
    def userId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.loginName")
    def loginName(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.firstName")
    def firstName(self): ...


class RoleRelUsers(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.payload.nextPage")
    def nextPage(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=RelUser)
    @cjen.operate.json.one(json_path="$.payload.list")
    def list(self) -> list[RelUser]: ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user.portal_find_user import PortalFindUser


class PortalCreateUser(StandardStep):
    Name = "portal_create_user.py"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self): PortalFindUser(self.service, self.scenario).run()

    def ignore(self):
        return self.service.context[PortalFindUser.Id] is not None

    def login_name(self): return self.scenario.user()

    def pass_word(self): return self.service.encypt_password("Admin@123")

    def data(self):
        return {
            "firstName": self.login_name()[0],
            "lastName": self.login_name()[1:],
            "loginName": self.login_name(),
            "password": self.pass_word(),
            "phoneNumber": "88888888",
            "email": f'{self.login_name()}@163.com', "active": True, "etmfAdministrator": False}

    def _execute(self):
        super()._execute()
        self.service.user_api_create_user(data=self.data())



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.design.design_api import Versioning
from eclinical.standard.steps.design.versioning.design_get_last_version import DesignGetLastVersion


# 获取生成文件的fileid
class DesignFileIDS(StandardStep):
    Name = "design_fileids.py"

    def __init__(self, service: Versioning, scenario: Scenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        DesignGetLastVersion(self.service, self.scenario).run()

    def _execute(self):  self.service.archive_files(path_variable=self.path_variable())

    def call_back(self, **kwargs):
        self.service.context["archives"] = kwargs.get("archives")

    def path_variable(self):
        return dict(currentVersionId=self.service.context["currentVersionId"])



import time

import cjen
import eclinical
import jsonpath
from cjen import BigTangerine

from eclinical.utils.ecrypto import Ecrypto


class _LoginService(BigTangerine):
    @cjen.headers.basicHeaders(headers={
        "Accept": "application/json, text/plain, */*",
        "Content-Type": "application/json"
    })
    def __init__(self, environment: eclinical.Environment):
        super().__init__()
        self.environment = environment
        self.base_url = self.__dict__.get("base_url", self.environment.uri)
        self.context["userName"] = self.context.get("userName") or self.environment.user
        self.context["password"] = self.context.get("password") or self.environment.password
        self.context["company"] = self.context.get('company') or self.environment.company
        self.context["sponsor"] = self.context.get("sponsor") or self.environment.sponsor
        self.context["study"] = self.context.get("study") or self.environment.study
        self.context["env"] = self.context.get("env") or self.environment.life_cycle

    @cjen.http.post_mapping(uri="admin/auth")
    @cjen.jwt(key="Authorization", json_path="$.payload.jwtAuthenticationResponse.token")
    def auth(self, data, resp=None, **kwargs): ...

    @cjen.http.get_mapping(uri="admin/user/onboard/company")
    def company(self, resp=None, **kwargs):
        self.context["company_id"] = \
            jsonpath.jsonpath(resp,
                              f"$.payload[?(@.name=='{self.context.get('company')}')].id")[
                0]

    def encypt_password(self, pwd):
        return Ecrypto(self.environment).en(repr(dict(password=pwd, time=int(time.time() * 1000))).replace("'", "\""))

    def user_pwd(self):
        return dict(userName=self.context.get("userName") or self.environment.user,
                    password=self.encypt_password(self.context.get("password") or self.environment.password))


from Crypto.Cipher import PKCS1_v1_5 as Cipher_pksc1_v1_5
from Crypto.PublicKey import RSA
import base64

# from eClinical import Environment
from eclinical.environment.environment import Environment


class Ecrypto:
    def __init__(self, environment: Environment):
        self.environment = environment

    def en(self, string):
        public_key = self.environment.public_key
        public_key = "-----BEGIN PUBLIC KEY-----\n" + public_key + "\n-----END PUBLIC KEY-----"
        rsa_key = RSA.importKey(public_key)
        cipher = Cipher_pksc1_v1_5.new(rsa_key)
        cipher_text = base64.b64encode(cipher.encrypt(string.encode()))
        return cipher_text.decode()



from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user.portal_find_user import PortalFindUser


class PortalUserAccess(StandardStep):
    Name = "portal_user_access.py"
    StudiesId = "user_access_permission_studies_id"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        PortalFindUser(self.service, self.scenario).run()

    def path_variable(self): return {"user_id": self.service.context[PortalFindUser.Id]}

    def _execute(self):
        super()._execute()
        self.service.user_api_user_access(path_variable=self.path_variable())

    def call_back(self, **kwargs):
        id_of_studies = set()
        for user_permission_dto in kwargs.get("user_access").userPermissionDtoList():
            id_of_studies.add(user_permission_dto.studyId())
        self.service.context[self.StudiesId] = list(id_of_studies)



import os

from eclinical.scaffold.inputs.mc.meta_service import MetaService
from eclinical.scaffold.template.file_operate import FileOperate
from eclinical.scaffold.template.meta import Meta
# from scaffold.template.meta import Meta


class GemMeta:
    def __init__(self, folder):
        self.folder = folder

    def add_meta(self, meta_name: str, meta_type: str, props: list[str]):
        meta = Meta(meta_name, props)
        fop = FileOperate(self.folder)

        service = self.belong_service()

        body = meta.json() if meta_type == 'Json' else meta.mysql()

        sub_folder = os.path.join("meta", service) if service is not None else "meta"
        if not os.path.exists(os.path.join(self.folder, sub_folder)):
            os.mkdir(os.path.join(self.folder, sub_folder))
        fop.rewrite_file(sub_folder, meta.name + ".py", body) \
        or fop.new_file(sub_folder, meta.name + ".py", body)

    @classmethod
    def ok_response(cls, folder: str):
        body = """
import cjen
from cjen import MetaJson


class OKResponse(MetaJson):
    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...
"""
        fop = FileOperate(folder)
        fop.rewrite_file("meta", "ok_response.py", body) \
        or fop.new_file("meta", "ok_response.py", body)

    def belong_service(self):
        services = ["不属于任何service"]
        for root, folder, files in os.walk(os.path.join(self.folder, "services")):
            for file in files:
                if file.endswith("_service.py"):
                    services.append(file.replace(".py", ""))
        service = MetaService.input(select=tuple(services))
        if service == "不属于任何service": return None
        return service



from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.ctms_scenario import CtmsScenario
from eclinical.standard.steps.ctms.ctms_all_menu import CtmsAllMenu
from eclinical.standard.steps.ctms.ctms_common_country import CtmsCommonCountry
from eclinical.standard.steps.ctms.ctms_find_site import CtmsFindSite


class CtmsCreateSiteEntity(StandardStep):
    Name = "ctms_create_site_entity.py"

    def __init__(self, service, scenario: CtmsScenario):
        super().__init__(service, scenario)

    def _pre_processor(self):
        CtmsCommonCountry(self.service, self.scenario).run()
        CtmsAllMenu(self.service, self.scenario).run()

    def code_of_menus(self):
        return ["menu.ctms_entity_management", "menu.ctms_entity_management_site"]

    def find_country_id(self):
        return self.service.context[CtmsCommonCountry.Country].get("Afghanistan")

    def find_menu_id(self, menus_code, menu_tree, level=0):
        if menu_tree is not None:
            for menu in menu_tree:
                if menu.get("code") == menus_code[level]:
                    if level + 1 == len(self.code_of_menus()): return menu.get("id")
                    return self.find_menu_id(menus_code, menu.get("children"), level + 1)
        raise Exception(f'can not find {self.code_of_menus()}')

    def sites(self):
        return self.scenario.sites()

    def ignore(self):
        new_sites = list(filter(lambda dict_item: dict_item[1] == CtmsFindSite.NoFill, self.service.context[CtmsFindSite.Info].items()))
        return len(new_sites) == 0

    def data(self):
        menu_id = self.find_menu_id(self.code_of_menus(), self.service.context[CtmsAllMenu.Menu])
        country_id = self.find_country_id()
        new_sites = dict(filter(lambda dict_item: dict_item[1] == CtmsFindSite.NoFill, self.service.context[CtmsFindSite.Info].items()))

        return [
            {"siteId": "null",
             "siteName": site_name,
             "postalOrZoneCode": "0",
             "countryId": f'{country_id}',
             "provinceOrState": "China",
             "city": "China",
             "phone": "88888888",
             "fax": "",
             "email": "",
             "address": "China",
             "active": "true",
             "menuId": f'{menu_id}'} for site_name, site_id in new_sites.items()]

    def _execute(self):
        super()._execute()
        for d in self.data():
            self.service.entity_management_add_site(data=d)



import argparse
import os

from eclinical.scaffold.files.gen_service import GenService
from eclinical.scaffold.inputs.application import Application

# from scaffold.files.gen_service import GenService
from eclinical.scaffold.inputs.sc.service_name import ServiceName

parser = argparse.ArgumentParser()
parser.add_argument("-name", help="创建或更新的 service 名")
args = parser.parse_args()


class Sc:
    def __init__(self, name):
        self.name = name
        self.folder = os.getcwd()

    def path_right(self):
        if not os.path.exists(os.path.join(self.folder, "services")):
            raise Exception("无法找到services文件夹")


def sc_command():
    try:
        name = args.name or ServiceName.input()
        sc = Sc(name)
        sc.path_right()
        GenService(sc.folder).add_service(name, Application.input(msg=f"{name}是什么系统"))
    except Exception as e:
        print(e)

# if __name__ == '__main__':
#     sc_command()



import argparse
import os

from eclinical.scaffold.files.gen_service import GenService
from eclinical.scaffold.inputs.application import Application
from eclinical.scaffold.inputs.ecl.module_name import ModuleName
from eclinical.scaffold.inputs.ecl.module_service import ModuleService
from eclinical.scaffold.inputs.ecl.module_type import ModuleType
from eclinical.scaffold.template.file_operate import FileOperate
from eclinical.scaffold.files.gen_meta import GemMeta
from eclinical.scaffold.template.service import Service

parser = argparse.ArgumentParser()
parser.add_argument("-init", action='store_true', help="初始化一个module，包括生成environment.yaml，全局的conftest")
parser.add_argument("-ns", action='store_true', help="执行初始化module时，如果带这个参数，表明创建Module 同时创建 service")
args = parser.parse_args()


class Ecl:

    def __init__(self, name=None, module_type=None):
        self.name = name
        self.module_type = module_type
        self.ecl_folder = os.path.join(os.getcwd(), f'{self.name}_{self.module_type}')

    def init_module(self):
        if os.path.exists(self.ecl_folder): raise Exception("请勿重复创建")
        os.mkdir(self.ecl_folder)
        os.mkdir(os.path.join(self.ecl_folder, "services"))
        os.mkdir(os.path.join(self.ecl_folder, "meta"))
        os.mkdir(os.path.join(self.ecl_folder, "ext"))
        FileOperate(self.ecl_folder).new_file("", "__init__.py", "")
        if self.module_type == "testcase":
            FileOperate(self.ecl_folder).new_file("", "conftest.py", Service.output_conftest_header_py())
            GemMeta.ok_response(self.ecl_folder)


def ecl_command():
    if args.init:
        ecl = Ecl(ModuleName.input(), ModuleType.input())
        ecl.init_module()
        if args.ns:
            for service_name in ModuleService.input():
                gen = GenService(ecl.ecl_folder)
                gen.add_service(service_name, Application.input(msg=f"{service_name}是什么系统"))
                # ecl.add_service(service_name, Application.input(msg=f"{service_name}是什么系统"))

# if __name__ == '__main__':
#     ecl_command()



import argparse
import os

from eclinical.scaffold.files.gen_meta import GemMeta
# from scaffold.files.gen_meta import GemMeta
from eclinical.scaffold.inputs.mc.meta_attrs import MetaAttrs
from eclinical.scaffold.inputs.mc.meta_name import MetaName
from eclinical.scaffold.inputs.mc.meta_type import MetaType

parser = argparse.ArgumentParser()
parser.add_argument("--name", help="创建meta 对象")
parser.add_argument("--type", help="meta 对象类型,mysql 或 json")
parser.add_argument("--attrs", help="对象属性，多个属性以,分割")

args = parser.parse_args()

# TODO 在 meta 中增加帮助示例
class Mc:

    def __init__(self):
        self.folder = os.getcwd()

    def meta(self, meta_name, meta_type, meta_attrs):
        GemMeta(self.folder).add_meta(meta_name, meta_type, meta_attrs)

    def right_path(self):
        if not os.path.exists(os.path.join(self.folder, "meta")):
            raise "没有找到meta文件夹,请确认是在module 文件夹下"


def mc_command():
    try:
        mc = Mc()
        mc.right_path()
        mc.meta(args.name or MetaName().input(),
                args.type or MetaType().input(),
                args.attrs or MetaAttrs().input())
    except Exception as e:
        print(e)


# if __name__ == '__main__':
#     mc_command()



from cjen.sco.scenario import Scenario


class CtmsScenario(Scenario):
    def __init__(self, scenario_dir, service=None):
        super().__init__(scenario_dir, service)

    def sponsor(self):
        return self.get("sponsor").get("name")

    def study(self):
        return self.get("study").get("name")

    def life_cycle(self):
        return self.get("life_cycle").get("env")

    def sites(self):
        return self.get("user_groups").get("sites")

    def apps(self): return self.get("study").get("apps")



import cjen
from cjen import MetaJson


class User(MetaJson):
    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.companyId")
    def companyId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.active")
    def active(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.loginName")
    def loginName(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...


class Users(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.payload.nextPage")
    def nextPage(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=User)
    @cjen.operate.json.one(json_path="$.payload.list")
    def list(self) -> list[User]: ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...



from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.ctms_scenario import CtmsScenario


class CtmsFindStudy(StandardStep):
    Name = "ctms_find_study.py"
    Info = "ctms_study_dto"

    def __init__(self, service, scenario: CtmsScenario):
        super().__init__(service, scenario)
        self.service.context[self.Info] = None

    def call_back(self, **kwargs):
        for study in kwargs.get("studies").list():
            if study.name() == self.scenario.study():
                self.service.context[self.Info] = dict(id=study.id(), name=study.name())

    def _execute(self):
        super()._execute()
        self.service.study_management_get_study()



import os

from eclinical import Environment
from eclinical.standard.ctms.ctms_api import CtmsApi
from eclinical.standard.scenarios.ctms_scenario import CtmsScenario
from eclinical.standard.steps.ctms.ctms_common_country import CtmsCommonCountry
from eclinical.standard.steps.ctms.ctms_create_site_entity import CtmsCreateSiteEntity
from eclinical.standard.steps.ctms.ctms_find_site import CtmsFindSite
from eclinical.standard.steps.ctms.ctms_find_study import CtmsFindStudy
from eclinical.standard.steps.ctms.ctms_find_study_site import CtmsFindStudySite
from eclinical.standard.steps.ctms.ctms_study_set_site import CtmsStudySetSite
from eclinical.standard.steps.ctms.ctms_submit_study_basic_info import CtmsSubmitStudyBasicInfo


class ScoCtmsStudy:
    def __init__(self, scenario_dir, environment: Environment):
        self.scenario = CtmsScenario(scenario_dir, CtmsApi(environment))
        # service =
        self.scenario.add_step(CtmsCommonCountry)
        self.scenario.add_step(CtmsFindStudy)
        self.scenario.add_step(CtmsSubmitStudyBasicInfo)
        self.scenario.add_step(CtmsFindSite)
        self.scenario.add_step(CtmsCreateSiteEntity)
        self.scenario.add_step(CtmsFindStudySite)
        self.scenario.add_step(CtmsStudySetSite)

    def run(self): self.scenario.run()


if __name__ == '__main__':
    sco_dir = os.path.join("new_study")
    envir = Environment.loader("DEV_03")
    sco = ScoCtmsStudy(sco_dir, envir)
    sco.run()



import cjen
from cjen.sco.step_definitions import StepDefinitions

from eclinical import PortalLoginService, Environment
from eclinical.standard.base.ok_response import OkResponse
from eclinical.standard.portal.dto.company_envs import CompanyEnvs
from eclinical.standard.portal.dto.envs import Envs
from eclinical.standard.portal.dto.group_user import GroupUsers
from eclinical.standard.portal.dto.hierarchy import Hierarchy
from eclinical.standard.portal.dto.permission_tree import PermissionTree
from eclinical.standard.portal.dto.query_studies import QueryStudies
from eclinical.standard.portal.dto.query_user_groups import QueryUserGroups
from eclinical.standard.portal.dto.querysponsors import QuerySponsors
from eclinical.standard.portal.dto.role_list import RoleList
from eclinical.standard.portal.dto.role_rel_user import RoleRelUsers
from eclinical.standard.portal.dto.systems import Systems
from eclinical.standard.portal.dto.user_access import UserAccess
from eclinical.standard.portal.dto.user_group_info import UserGroupInfo
from eclinical.standard.portal.dto.user_group_role_rel import UserGroupRoleRel
from eclinical.standard.portal.dto.user_roles import UserRoles
from eclinical.standard.portal.dto.users import Users
from eclinical.standard.steps.portal.hierarchies.portal_find_sponsor import PortalFindSponsor
from eclinical.standard.steps.portal.hierarchies.portal_find_study import PortalFindStudy
from eclinical.standard.steps.portal.hierarchies.portal_sponsor_no_start import PortalSponsorNoStart
from eclinical.standard.steps.portal.hierarchies.portal_study_no_start import PortalStudyNoStart
from eclinical.standard.steps.portal.role.portal_find_role import PortalFindRole
from eclinical.standard.steps.portal.role.portal_get_permission_tree import PortalGetPermissionTree
from eclinical.standard.steps.portal.user.portal_associated_usergroup import PortalAssociatedUserGroup
from eclinical.standard.steps.portal.user.portal_find_user import PortalFindUser
from eclinical.standard.steps.portal.user.portal_find_user_role import PortalFindUserRole
from eclinical.standard.steps.portal.user_groups.portal_find_un_added_user import PortalFindUnAddedUser
from eclinical.standard.steps.portal.user_groups.portal_find_usergroup import PortalFindUserGroup
from eclinical.standard.steps.portal.user_groups.portal_get_company_envs import PortalGetCompanyEnvs
from eclinical.standard.steps.portal.user_groups.portal_get_lifecycle_hierarchies import PortalGetLifeCycleHierarchies
from eclinical.standard.steps.portal.user_groups.portal_get_user_group_info import PortalGetUserGroupInfo
from eclinical.standard.steps.portal.user_groups.portal_life_cycle import PortalLifeCycle
from eclinical.standard.steps.portal.user_roles.portal_find_no_rel_user import PortalFindNoRelUser
from eclinical.standard.steps.portal.user_roles.portal_get_user_role import PortalGetUserRole


class PortalApi(PortalLoginService):
    def __init__(self, environment: Environment = None):
        super().__init__(environment)
        self.step_definitions = StepDefinitions()

    @cjen.http.get_mapping(uri="admin/company/envs", json_clazz=Envs)
    @cjen.operate.asserts.validation_meta(meta_name="envs")
    @cjen.step.call(stepName=PortalLifeCycle.Name, argName="envs")
    def hierarchies_get_env_list(self, envs: Envs = None, resp=None, **kwargs):
        ...

    @cjen.http.get_mapping(uri="admin/company/env/{env_id}/hierarchies", json_clazz=QuerySponsors)
    @cjen.operate.asserts.validation_meta(meta_name="query")
    @cjen.step.call(stepName=PortalFindSponsor.Name, argName="query")
    def hierarchies_get_sponsor(self, path_variable, query: QuerySponsors = None, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/study/query?pageNo={pageNo}&pageSize=25", json_clazz=QueryStudies)
    @cjen.step.call(stepName=PortalFindStudy.Name, argName="query")
    def hierarchies_get_study(self, path_variable, data, query: QueryStudies = None, resp=None, **kwargs):
        if not self.context[PortalFindStudy.Id]:
            if not query.nextPage() == 0:
                self.hierarchies_get_study(path_variable=dict(pageNo=query.nextPage()), data=data)

    @cjen.http.post_mapping(uri="admin/sponsor", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok")
    def hierarchies_new_sponsor(self, data, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/study", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok")
    def hierarchies_new_study(self, data, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.get_mapping(uri="admin/sponsor/{sponsor_id}/env/{env_id}/systems", json_clazz=Systems)
    @cjen.step.call(stepName=PortalSponsorNoStart.Name, argName="systems")
    def hierarchies_sponsor_systems(self, path_variable, systems: Systems = None, resp=None, **kwargs):
        ...

    @cjen.http.get_mapping(uri="admin/study/{study_id}/env/{env_id}/systems", json_clazz=Systems)
    @cjen.step.call(stepName=PortalStudyNoStart.Name, argName="systems")
    def hierarchies_study_systems(self, path_variable, systems: Systems = None, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/study/{study_id}/env/{env_id}/database", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok")
    def hierarchies_study_startup(self, path_variable, data, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/sponsor/{sponsor_id}/env/{env_id}/database", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok")
    def hierarchies_sponsor_startup(self, path_variable, data, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.put_mapping(uri="admin/role/{role_id}/permissions", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok", fields="procCode")
    def role_api_set_permissions(self, path_variable, data, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.get_mapping(uri="admin/role/{role_id}/allPermissionTree", json_clazz=PermissionTree)
    @cjen.step.call(stepName=PortalGetPermissionTree.Name, argName="pt")
    def role_api_all_permission_tree(self, path_variable, pt: PermissionTree = None, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/role")
    def role_api_create_role(self, data, resp=None, **kwargs):
        ...

    @cjen.http.get_mapping(uri="admin/role/category/list", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok", fields="procCode")
    def role_api_category_list(self, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/role/query?pageNo={pageNo}&pageSize=25", json_clazz=RoleList)
    @cjen.step.call(stepName=PortalFindRole.Name, argName="role_list")
    def role_api_query(self, path_variable, role_list: RoleList, resp=None, **kwargs):
        ...

    @cjen.http.put_mapping(uri="admin/user/{user_id}/study_title?sendEmail=false", json_clazz=OkResponse)
    def user_api_study_title(self, path_variable, data, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.get_mapping(uri="admin/user/{user_id}/access_list", json_clazz=UserAccess)
    def user_api_user_access(self, path_variable, user_access: UserAccess = None, resp=None, **kwargs):
        from eclinical.standard.steps.portal.user.portal_user_access import PortalUserAccess
        self.step_definitions.call_back(PortalUserAccess.Name, user_access=user_access)

    @cjen.http.get_mapping(uri="admin/role/list", json_clazz=RoleList)
    @cjen.operate.asserts.validation_meta(meta_name="role_list", fields="procCode")
    @cjen.step.call(stepName=PortalFindUserRole.Name, argName="role_list")
    def user_api_role_list(self, role_list: RoleList = None, resp=None, **kwargs):
        ...

    @cjen.http.put_mapping(uri="admin/role/{user_id}/user_group_role_rel", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok", fields="procCode")
    def user_api_append_user_group_role_rel(self, path_variable, data, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.get_mapping(uri="admin/user/{user_id}/user_group_role_rel", json_clazz=UserGroupRoleRel)
    @cjen.operate.asserts.validation_meta(meta_name="rel", fields="procCode")
    @cjen.step.call(stepName=PortalAssociatedUserGroup.Name, argName="rel")
    def user_api_user_group_role_rel(self, path_variable, rel: UserGroupRoleRel, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/user/query?pageNo={pageNo}&pageSize=25", json_clazz=Users)
    @cjen.step.call(stepName=PortalFindUser.Name, argName="users")
    def user_api_user_query(self, path_variable, users: Users = None, resp=None, **kwargs):
        if not self.context[PortalFindUser.Id]:
            if not users.nextPage() == 0:
                self.user_api_user_query(path_variable=dict(pageNo=users.nextPage()))

    @cjen.http.post_mapping(uri="admin/user", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok", fields="procCode")
    def user_api_create_user(self, data, ok: OkResponse, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/userGroup/query?pageNo={pageNo}&pageSize=25", json_clazz=QueryUserGroups)
    @cjen.step.call(stepName=PortalFindUserGroup.Name, argName="users")
    def user_get_user_group(self, path_variable, query: QueryUserGroups = None, resp=None, **kwargs):
        if not self.context[PortalFindUserGroup.Id]:
            if not query.nextPage() == 0:
                self.user_get_user_group(path_variable=dict(pageNo=query.nextPage()))

    @cjen.http.put_mapping(uri="admin/company/env/{life_cycle_id}/user-group/{userGroup_id}/hierarchies", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok", fields="procCode")
    def usergroups_api_set_lifecycle_hierarchies(self, path_variable, data, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.get_mapping(uri="admin/company/env/{life_cycle_id}/user-group/{userGroup_id}/hierarchies",
                           json_clazz=Hierarchy)
    @cjen.step.call(stepName=PortalGetLifeCycleHierarchies.Name, argName="hierarchies")
    def usergroups_api_get_lifecycle_hierarchies(self, path_variable, hierarchies: Hierarchy, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/userGroup/query?pageNo={pageNo}&pageSize=25", json_clazz=QueryUserGroups)
    @cjen.step.call(stepName=PortalFindUserGroup.Name, argName="query")
    def usergroups_get_user_group(self, path_variable, query: QueryUserGroups = None, resp=None, **kwargs):
        if not self.context[PortalFindUserGroup.Id]:
            if not query.nextPage() == 0:
                self.usergroups_get_user_group(path_variable=dict(pageNo=query.nextPage()))

    @cjen.http.get_mapping(uri="admin/userGroup/{userGroup_id}", json_clazz=UserGroupInfo)
    @cjen.step.call(stepName=PortalGetUserGroupInfo.Name, argName="query")
    def usergroups_user_group_info(self, path_variable, query: UserGroupInfo = None, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/userGroup", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok", fields="procCode")
    def usergroups_create_user_group(self, data, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/user")
    def usergroups_create_user(self, data, resp=None, **kwargs):
        ...

    @cjen.http.get_mapping(uri="admin/company/envs", json_clazz=CompanyEnvs)
    @cjen.step.call(stepName=PortalGetCompanyEnvs.Name, argName="company_envs")
    def usergroups_api_company_envs(self, company_envs: CompanyEnvs = None, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/userGroup/{userGroup_id}/unAddedUsers?pageNo=1&pageSize=25",
                            json_clazz=GroupUsers)
    @cjen.step.call(stepName=PortalFindUnAddedUser.Name, argName="group_users")
    def usergroups_api_get_un_added_user(self, path_variable, group_users: GroupUsers = None, resp=None, **kwargs):
        if not self.context["user_group_id"]:
            if not group_users.nextPage() == 0:
                self.usergroups_get_user_group(path_variable=dict(pageNo=group_users.nextPage()))

    @cjen.http.post_mapping(uri="admin/userGroup/{userGroup_id}/add/users", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok", fields="procCode")
    def usergroups_api_add_group_user(self, path_variable, data, ok: OkResponse = None, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/role/{role_id}/user_group_user")
    def userrole_api_user_group_user(self, path_variable, data, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/role/{role_id}/relation/user_group_users?pageNo={pageNo}&pageSize=25")
    def userrole_get_relations_users(self, path_variable, data, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="admin/role/{role_id}/no_relation/user_group_users?pageNo={pageNo}&pageSize=25", json_clazz=RoleRelUsers)
    @cjen.step.call(stepName=PortalFindNoRelUser.Name, argName="users")
    def userrole_get_no_relations_users(self, path_variable, data, users: RoleRelUsers, resp=None, **kwargs):
        if not self.context[PortalFindNoRelUser.Id]:
            if not users.nextPage() == 0:
                self.userrole_api_get_user_roles(path_variable=dict(pageNo=users.nextPage(), role_id=path_variable.get("role_id")))

    @cjen.http.get_mapping(uri="admin/role/user_roles?pageNo={pageNo}&pageSize=25", json_clazz=UserRoles)
    @cjen.step.call(stepName=PortalGetUserRole.Name, argName="user_role")
    def userrole_api_get_user_roles(self, path_variable, user_role: UserRoles = None, resp=None, **kwargs):
        if not self.context[PortalGetUserRole.Id]:
            if not user_role.nextPage() == 0:
                self.userrole_api_get_user_roles(path_variable=dict(pageNo=user_role.nextPage()))



import cjen
from cjen import MetaJson


class UserPermissionDto(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.sponsorId")
    def sponsorId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.studyId")
    def studyId(self): ...


class UserAccess(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=UserPermissionDto)
    @cjen.operate.json.one(json_path="$.payload.userPermissionDtoList")
    def userPermissionDtoList(self) -> list[UserPermissionDto]: ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...



import winreg
import ctypes


class Proxy:

    def __init__(self):
        self.internet_settings = None
        self.init_internet_settings()

    def set_key(self, name, value, reg_type):
        winreg.SetValueEx(self.internet_settings, name, 0, reg_type, value)

    def init_internet_settings(self):
        try:
            self.internet_settings = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                                                    r'Software\Microsoft\Windows\CurrentVersion\Internet Settings',
                                                    0, winreg.KEY_ALL_ACCESS)
        except OSError as e:
            print(e)

    def open(self):
        # 启用代理
        self.set_key('ProxyEnable', 1, winreg.REG_DWORD)  # 启用
        self.set_key('ProxyOverride', u'*.local;<local>', winreg.REG_SZ)  # 绕过本地
        self.set_key('ProxyServer', u'127.0.0.1:9999', winreg.REG_SZ)  # 代理IP及端口，将此代理修改为自己的代理IP

    def close(self):
        self.set_key('ProxyEnable', 0, winreg.REG_DWORD)  # 停用


win_proxy = Proxy()



import os

from eclinical.scaffold.template.file_operate import FileOperate
from eclinical.scaffold.template.service import Service
# from scaffold.template.service import Service


class GenService:
    def __init__(self, folder):
        self.folder = folder

    def add_service(self, service_names: str, app: str):
        for service_name in service_names.split(","):
            service = Service(service_name, app)
            # 输出到service.py
            if not os.path.exists(os.path.join(self.folder, "services")): os.mkdir(
                os.path.join(self.folder, "services"))
            fop = FileOperate(self.folder)
            if fop.file_exist("services", service.file_name):
                if input(f"{service.service_name} 已存在，是否覆盖(Y/N)") == 'N':
                    continue
            fop.rewrite_file("services", service.file_name + ".py", service.output_service_py()) \
            or fop.new_file("services", service.file_name + ".py", service.output_service_py())
            # 输出到 __init__.py
            fop.append_file("services", "__init__.py", service.output_init_py()) \
            or fop.new_file("services", "__init__.py", service.output_init_py())
            # 输出到 conftest.py
            fop.append_file("", "conftest.py", service.output_conftest_body_py()) \
            or fop.new_file("", "conftest.py", service.output_conftest_body_py())




from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.role.portal_find_role import PortalFindRole


class PortalGetPermissionTree(StandardStep):
    Name = "portal_get_permission_tree.py"
    IdOfAllPermission = "all_permissions_ids"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def _pre_processor(self):
        PortalFindRole(self.service, self.scenario).run()

    def path_variable(self):
        return dict(role_id=self.service.context[PortalFindRole.Id])

    def _execute(self):
        super()._execute()
        self.service.role_api_all_permission_tree(path_variable=self.path_variable())

    def call_back(self, **kwargs):
        ids = set()
        for system in kwargs.get("pt").systems():
            self.get_permission_id(system, ids)
        self.service.context[self.IdOfAllPermission] = list(ids)

    def get_permission_id(self, node, cache):
        for menu in node.get("children"):
            cache.add(menu.get("id"))
            if menu.get("children"):
                self.get_permission_id(menu, cache)



import cjen
import re


class Swagger(cjen.BigTangerine):
    def __init__(self):
        self.__setattr__("base_url", "http://200.200.101.113")
        super().__init__()

    @cjen.http.get_mapping(uri="api/{system}/v2/api-docs")
    def doc(self, path_variable, resp=None, **kwargs):
        ...

    @classmethod
    def with_param(cls, key):
        if re.findall(r'{([^}]+)}', key):
            return True
        return False

    def apis(self, system):
        try:
            apis = list(filter(self.with_param, self.doc(path_variable=dict(system=system)).get("paths").keys()))
            return (key for key in apis)
        except Exception as e:
            return []


if __name__ == '__main__':
    print(Swagger().apis("design"))



from eclinical.standard.portal.dto.hierarchies.node import Node
from eclinical.standard.portal.dto.hierarchies.site import Site
from eclinical.standard.portal.dto.hierarchy import StudyDto


class Study(Node):
    def __init__(self, value: StudyDto, parent: Node, relationObjectId):
        super().__init__(value, parent, relationObjectId)
        self.level = 2
        self.sponsorId = None
        self.siteList = [Site(site_dto, self, relationObjectId) for site_dto in value.siteList()]

    def selected(self, relationObjectId):
        super(Study, self).selected(relationObjectId)
        self.parent.selected(relationObjectId)

    # Study 维度选择
    def top_selected(self):
        self.hasRelation = True
        # 选中上级Sponsor
        self.parent.selected()
        # 选中该study下的所有Site
        for site in self.siteList:
            site.__top_selected()

    def to_dict(self):
        result = super(Study, self).to_dict()
        result["siteList"] = [study.to_dict() for study in self.siteList]
        return result



import os

from eclinical import Environment
from eclinical.standard.portal.portal_api import PortalApi
from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.role.portal_create_role import PortalCreateRole
from eclinical.standard.steps.portal.role.portal_find_role import PortalFindRole
from eclinical.standard.steps.portal.role.portal_get_permission_tree import PortalGetPermissionTree
from eclinical.standard.steps.portal.role.portal_set_permissions import PortalSetPermissions


class ScoCreateRole:
    def __init__(self, scenario_dir, environment: Environment):
        self.scenario = PortalScenario(scenario_dir)
        self.service = PortalApi(environment)
        # 查询角色
        self.scenario.append_step(PortalFindRole, self.service)
        # 查询创建角色
        self.scenario.append_step(PortalCreateRole, self.service)
        # 获取菜单树
        self.scenario.append_step(PortalGetPermissionTree, self.service)
        # 为角色设置所有的菜单权限
        self.scenario.append_step(PortalSetPermissions, self.service)

    def run(self): self.scenario.run()


if __name__ == '__main__':
    sco_dir = os.path.join("new_study")
    envir = Environment.loader("DEV_03")
    sco = ScoCreateRole(sco_dir, envir)
    sco.run()


import cjen

from eclinical.environment.environment import Environment
from eclinical.service._study_login_service import _StudyLoginService


class IWRSLoginService(_StudyLoginService):
    @cjen.context.add(content=dict(system="IWRS"))
    def __init__(self, environment: Environment = None):
        super().__init__(environment)
        self.study_auth()

    @cjen.http.post_mapping(uri="iwrs/auth")
    @cjen.jwt(key="Authorization", json_path="$.payload.jwtAuthenticationResponse.token")
    def study_auth(self, resp=None, **kwargs):
        ...


import cjen

from eclinical.environment.environment import Environment
from eclinical.service._login_service import _LoginService


class PortalAdministratorLoginService(_LoginService):
    def __init__(self, environment: Environment):
        super().__init__(environment)
        self.auth(data=self.user_pwd())

    @cjen.http.post_mapping(uri="admin/administrator/auth")
    @cjen.jwt(key="Authorization", json_path="$.payload.jwtAuthenticationResponse.token")
    def auth(self, data, resp=None, **kwargs): ...


from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.ctms_scenario import CtmsScenario


class CtmsFindSite(StandardStep):
    Name = "ctms_find_site.py"
    Info = "ctms_site_entyties_id"
    NoFill = -1

    def __init__(self, service, scenario: CtmsScenario):
        super().__init__(service, scenario)
        self.service.context[self.Info] = dict.fromkeys(self.sites(), self.NoFill)

    def sites(self):
        return self.scenario.sites()

    def ignore(self):
        try:
            for key, value in self.service.context[self.Info].items():
                if value == self.NoFill: return False
            return True
        except Exception as e:
            return False

    def need_fill(self, site_entity):
        return self.service.context[self.Info].get(site_entity.siteName()) is not None \
               and self.service.context[self.Info].get(site_entity.siteName()) == self.NoFill

    def call_back(self, **kwargs):
        for site_entity in kwargs.get("site_entities").list():
            if self.need_fill(site_entity):
                self.service.context[self.Info][site_entity.siteName()] = site_entity.id()

    def _execute(self):
        super()._execute()
        self.service.entity_management_site_list(path_variable=self.path_variable())

    def path_variable(self):
        return dict(pageNo=1)



from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user.portal_find_user import PortalFindUser
from eclinical.standard.steps.portal.user.portal_user_access import PortalUserAccess


class PortalStudyTitle(StandardStep):
    Name = "portal_study_title.py"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self): ...

    def path_variable(self):
        return dict(user_id=self.service.context[PortalFindUser.Id])

    def data(self):
        return [dict(studyId=study_id,
                     title=f'Title Of {study_id}',
                     userId=self.service.context[PortalFindUser.Id])
                for study_id in self.service.context[PortalUserAccess.StudiesId]]

    def _execute(self):
        super()._execute()
        self.service.user_api_study_title(path_variable=self.path_variable(), data=self.data())



# from scaffold.command.edk import Edk
from eclinical.scaffold.inputs.component.text import Text


class EnvironmentName:
    @classmethod
    def input(cls, edk, msg: str = "请输入 environment 名", default=None, default_key=None):
        text = Text(msg, default, default_key)
        name = text.get(input(f"{text}\n"))
        if edk.environment_exist(name):
            return cls.input(edk, "environment 名已存在,请重新输入", default, default_key)
        else:
            return name





SPACE4 = "    "
class Service:
    def __init__(self, service_name, app):
        self.service_name = service_name
        self.app = app

    @property
    def file_name(self):
        return f"{self.service_name}_service"

    @property
    def clazz_name(self):
        return "".join([elem.capitalize() for elem in self.file_name.split("_")])

    @property
    def application(self):
        return self.app

    @property
    def application_clazz(self):
        clazz = dict(etmf="ETMFLoginService", pv="PVLoginService", ctms="CTMSLoginService", edc="EdcLoginService",
                     iwrs="IWRSLoginService", design="DesignLoginService", portal="PortalLoginService",
                     portaladmin="PortalAdministratorLoginService").get(self.application.lower(), "BigTangerine")
        return clazz

    @classmethod
    def output_conftest_header_py(cls):
        return "import pytest\n\n" + \
               "from environment.environment import Environment\n\n"

    def output_conftest_body_py(self):
        return f"from .services.{self.file_name} import {self.clazz_name}\n\n\n" + \
               f"@pytest.fixture\n" + f"def {self.file_name}(ef, st):\n" + \
               f"\treturn {self.clazz_name}(Environment(envir=st, file_path=ef))\n\n\n"

    def output_init_py(self):
        return f"from .{self.file_name} import {self.clazz_name}\n"

    def output_service_py(self):

        if self.application_clazz == "BigTangerine":
            import_str = "import cjen\n\nfrom cjen import BigTangerine,DatabasePool\nfrom eclinical import Environment\n\n\n"
            super_str = "        super().__init__()\n"
        else:
            import_str = f"import cjen\n\nfrom cjen import DatabasePool\nfrom eclinical import {self.application_clazz}, Environment\n\n\n"
            super_str = "        super().__init__(environment)\n"
        return import_str + \
               f"class {self.clazz_name}({self.application_clazz}):\n" + \
               "    \"\"\"\n    相关使用详情可查询:\n" \
               "    http 装饰器: https://github.com/thcpc/cjen#432-http-%E8%A3%85%E9%A5%B0%E5%99%A8\n" +\
               "    数据构造 装饰器: https://github.com/thcpc/cjen#433-factory-%E8%A3%85%E9%A5%B0%E5%99%A8\n " + \
               "    \"\"\"\n\n" + \
               f"    def __init__(self, environment: Environment):\n" + \
               super_str + \
               f"        self.context['cursor'] = DatabasePool.cursor(host=environment.db.get(\"host\"), user=environment.db.get(\"user\"), pwd=environment.db.get(\"pwd\"), port=environment.db.get(\"port\"), database=environment.db.get(\"database\"))\n\n"

    def _method_name(self, api_url): pass

    def _method_body(self, api_url, flow):
        return f'{SPACE4}def  {self._method_name(api_url)}({self._method_args(api_url,flow)}, resp=None, **kwargs): ...\n'

    def _decorator_http_name(self,flow): return "post_mapping"

    def _method_decorator(self, api_url, flow):
        return f'{SPACE4}@cjen.http.{self._decorator_http_name(flow)}(uri={api_url})'

    def _method_args(self, api_url, flow):
        if '{' in api_url and flow.request.body is not None: return "path_variable, data"
        if '{' in api_url: return "path_variable"
        if flow.request.body is not None: return "data"

    def output_method(self, api_url, flow):
        return f'{self._method_decorator(api_url, flow)}\n{self._method_body(api_url, flow)}\n'

        # annoation = f'    @cjen.http.post(uri={api_url})'
        # method = f'    def {}'


import re
from urllib.parse import urlparse, parse_qs, urlunparse

from eclinical.record.ext.api.swagger import Swagger


class ApiSpec:

    def __init__(self, resource, application):
        self.application = application
        self.apis = resource.apis(self.application)
        self.hava_data = False
        self.have_path_variable = False

    @classmethod
    def api(cls, resource, http):
        application = http.path.split("/")[2]
        return ApiSpec(resource, application).spec(http=http)

    def spec(self, http):
        # 只能识别 ID 的接口
        http_path = "/".join(http.path.split("/")[2:])
        return self.parameters_in_query(http_path) or self.parameters_in_path(http_path) or http_path

    def parameters_in_path(self, http_path):
        for api in self.apis:
            api_regex = re.sub(r'\{\w+[Id | id | ID]\}', '\\\d+', f'{self.application}{api}')
            api_regex = re.sub(r'\{env}', '\\\d+', api_regex)
            api_regex = re.sub(r'\{location\}', '\\\d+', api_regex)
            try:
                if re.match(api_regex, http_path):
                    self.have_path_variable = True
                    return f'{self.application}{api}'
            except Exception as e:
                continue
        return None

    def parameters_in_query(self, http_path):
        result = urlparse(http_path)
        if  result.query:
            query = "&".join([f'{key}={{{key}}}' for key in parse_qs(result.query).keys()])
            self.have_path_variable = True
            return urlunparse(["", "", result.path, result.params, query, result.fragment])
        return None


if __name__ == '__main__':
    # print(ApiSpec(Swagger(), "admin").spec("admin/user/onboard/applications?companyId=1"))
    result = urlparse("http://200.200.101.113/api")
    print(result.hostname)


from eclinical.environment.environment import Environment
from eclinical.service.design_login_service import DesignLoginService
from eclinical.service.edc_login_service import EdcLoginService
from eclinical.service.iwrs_login_service import IWRSLoginService
from eclinical.service.portal_administrator_login_service import PortalAdministratorLoginService
from eclinical.service.ctms_login_service import CTMSLoginService
from eclinical.service.etmf_login_service import ETMFLoginService
from eclinical.service.portal_login_service import PortalLoginService
from eclinical.service.pv_login_service import PVLoginService
from eclinical.environment.smoke_environment import SmokeEnvironment
from eclinical.environment.function_environment import FunctionEnvironment

__all__ = [
    "DesignLoginService",
    "EdcLoginService",
    "IWRSLoginService",
    "PortalAdministratorLoginService",
    "CTMSLoginService",
    "ETMFLoginService",
    "PortalLoginService",
    "PVLoginService",
    "SmokeEnvironment",
    "FunctionEnvironment",
    "Environment"
]



from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.ctms_scenario import CtmsScenario


class CtmsAllMenu(StandardStep):
    Name = "ctms_all_menu.py"
    Menu = "ctms_all_menu"

    def __init__(self, service, scenario: CtmsScenario):
        super().__init__(service, scenario)



    def ignore(self):
        try:
            if self.service.context[self.Menu] is not None:
                return True
        except Exception as e:
            return False

    def call_back(self, **kwargs):
        self.service.context[self.Menu] = kwargs.get("menus").payload()

    def _execute(self):
        super()._execute()
        self.service.common_menu()



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user_groups.portal_life_cycle import PortalLifeCycle


class PortalFindSponsor(StandardStep):
    Name = "portal_find_sponsor"
    Id = "sponsor_id"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def sponsor(self):
        return self.scenario.sponsor()

    def _pre_processor(self):
        PortalLifeCycle(self.service, self.scenario).run()

    def _execute(self):
        super()._execute()
        self.service.hierarchies_get_sponsor(name=self.sponsor(), path_variable=self.path_variable())

    def call_back(self, **kwargs):
        self.service.context[self.Id] = None
        for sponsor in kwargs.get("query").sponsorExtDtoList():
            if sponsor.name() == self.sponsor():
                self.service.context[self.Id] = sponsor.id()

    def life_cycle(self):
        return self.scenario.life_cycle()

    def path_variable(self):
        return dict(env_id=self.service.context[self.life_cycle()])



import wx
import wx.xrc


class AlertDialog(wx.Dialog):
    def __init__(self, parent, msg):
        wx.Dialog.__init__(self, parent, id=wx.ID_ANY, title=u"错误提示", pos=wx.DefaultPosition,
                           size=wx.Size(200, 100), style=wx.DEFAULT_DIALOG_STYLE)

        self.SetSizeHints(wx.DefaultSize, wx.DefaultSize)

        bSizer1 = wx.BoxSizer(wx.VERTICAL)

        self.m_staticText3 = wx.StaticText(self, wx.ID_ANY, msg, wx.DefaultPosition, wx.DefaultSize, 0)
        self.m_staticText3.Wrap(-1)

        bSizer1.Add(self.m_staticText3, 0, wx.ALL, 5)

        self.m_button3 = wx.Button(self, wx.ID_ANY, u"关闭", wx.DefaultPosition, wx.DefaultSize, 0)
        bSizer1.Add(self.m_button3, 0, wx.ALL, 5)

        self.SetSizer(bSizer1)
        self.Layout()

        self.Centre(wx.BOTH)
        self.m_button3.Bind(wx.EVT_LEFT_UP, self.close_dialog)

    def __del__(self):
        pass

    def close_dialog(self, event):
        self.Close(True)
        self.GetParent().Enable()


    @classmethod
    def open(cls, parent, msg):
        if parent is not None: parent.Disable()
        error = AlertDialog(parent, msg)
        error.Show()

# if __name__ == '__main__':
#     app = wx.App()
#     main_win = AlertDialog(None, )
#     main_win.Show()
#     app.MainLoop()


from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user.portal_associated_usergroup import PortalAssociatedUserGroup
from eclinical.standard.steps.portal.user.portal_find_user import PortalFindUser
from eclinical.standard.steps.portal.user.portal_find_user_role import PortalFindUserRole
from eclinical.standard.steps.portal.user_groups.portal_find_usergroup import PortalFindUserGroup


class PortalUserAppendUserGroup(StandardStep):
    Name = "portal_user_append_usergroup.py"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        PortalFindUserGroup(self.service, self.scenario).run()
        PortalAssociatedUserGroup(self.service, self.scenario).run()

    def ignore(self):
        for rels in self.service.context[PortalAssociatedUserGroup.ExistsRels]:
            if self.role_eq(rels) and self.user_group_eq(rels) and self.user_eq(rels):
                return True
        return False

    def role_eq(self, rels):
        return rels.get("roleId") == self.service.context[PortalFindUserRole.Id]

    def user_eq(self, rels):
        return rels.get("userId") == self.service.context[PortalFindUser.Id]

    def user_group_eq(self, rels):
        return rels.get("userGroupId") == self.service.context[PortalFindUserGroup.Id]

    def path_variable(self):
        return dict(user_id=self.service.context[PortalFindUser.Id])

    def data(self):
        rels = self.service.context[PortalAssociatedUserGroup.ExistsRels]
        rels.append(dict(roleId=self.service.context[PortalFindUserRole.Id],
                         userId=self.service.context[PortalFindUser.Id],
                         userGroupId=self.service.context[PortalFindUserGroup.Id]))
        return rels

    def _execute(self):
        super()._execute()
        self.service.user_api_append_user_group_role_rel(path_variable=self.path_variable(), data=self.data())




from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario


class PortalFindRole(StandardStep):
    Name = "portal_find_role.py"
    Id = "role_id"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def path_variable(self): return dict(pageNo=1)

    def role_name(self):
        return self.scenario.role()

    def call_back(self, **kwargs):
        self.service.context[self.Id] = None
        for role in kwargs.get("role_list").all():
            if role.code() == self.role_name():
                self.service.context[self.Id] = role.id()

    def _execute(self):
        super()._execute()
        self.service.role_api_query(path_variable=self.path_variable())



import cjen
from cjen import MetaJson


class Role(MetaJson):
    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.active")
    def active(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.code")
    def code(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...


class RoleList(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=Role)
    @cjen.operate.json.one(json_path="$.payload.list")
    def all(self) -> list[Role]: ...

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=Role)
    @cjen.operate.json.one(json_path="$.payload")
    def payload(self) -> list[Role]: ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user_roles.portal_get_user_role import PortalGetUserRole


class PortalFindNoRelUser(StandardStep):
    Name = "portal_find_no_rel_user.py"
    Id = "No_REL_USER"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def _pre_processor(self):
        PortalGetUserRole(self.service, self.scenario).run()

    def path_variable(self):
        return dict(role_id=self.service.context[PortalGetUserRole.Id], PageNo=1)

    def data(self):
        return dict(role_id=self.service.context[PortalGetUserRole.Id])

    def user_name(self):
        return self.scenario.user().upper()

    def _execute(self):
        super()._execute()
        self.service.userrole_get_no_relations_users(path_variable=self.path_variable(), data=self.data())

    def call_back(self, **kwargs):
        self.service.context[self.Id] = None
        for user in kwargs.get("users").list():
            if user.loginName() == self.user_name():
                self.service.context[self.Id] = user.id()



import os

from cjen.sco.scenario import Scenario

from eclinical import Environment
from eclinical.standard.design.design_api import Versioning
from eclinical.standard.steps.design.versioning.design_download_publish_sql import DesignDownloadPublishSql
from eclinical.standard.steps.design.versioning.design_fileids import DesignFileIDS

from eclinical.standard.steps.design.versioning.design_get_last_version import DesignGetLastVersion


# 下载发布的SQL文件
class ScoDownLoadSql:
    def __init__(self, scenario_dir, environment: Environment):
        self.scenario = Scenario(scenario_dir)
        service = Versioning(environment)
        self.scenario.append_step(DesignDownloadPublishSql, service)
        # self.scenario.append_step(PortalStartupStudy, service)

    def run(self): self.scenario.run()


if __name__ == '__main__':
    sco_dir = os.path.join("new_study")
    sco = ScoDownLoadSql(sco_dir, Environment.loader("DEV_03"))
    sco.run()



import os


class FileOperate:
    def __init__(self, folder):
        self.folder = folder

    def rewrite_file(self, sub_folder, file_name, content):
        if self.file_exist(sub_folder, file_name):
            re_write = input(f"{file_name} 已存在,是否覆盖(Y/N)")
            if re_write.lower() == 'y':
                with open(os.path.join(self.folder, sub_folder, file_name), 'w', encoding="utf-8") as f:
                    f.write(content)
            return True
        return False

    def new_file(self, sub_folder, file_name, content):
        with open(os.path.join(self.folder, sub_folder, file_name), 'w', encoding="utf-8") as f:
            f.write(content)
        return True

    def append_file(self, sub_folder, file_name, content):
        if self.file_exist(sub_folder, file_name):
            with open(os.path.join(self.folder, sub_folder, file_name), 'a', encoding="utf-8") as f:
                f.write(content)
            return True
        return False

    def file_exist(self, sub_folder, file_name):
        return os.path.exists(os.path.join(self.folder, sub_folder, file_name))



import cjen
from cjen import MetaJson


class SiteDto(MetaJson):
    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.hasRelation")
    def hasRelation(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.code")
    def code(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.name")
    def name(self): ...


class StudyDto(MetaJson):
    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=SiteDto)
    @cjen.operate.json.one(json_path="$.siteList")
    def siteList(self) -> list[SiteDto]: ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.hasRelation")
    def hasRelation(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.name")
    def name(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...


class SponsorExtDto(MetaJson):
    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=StudyDto)
    @cjen.operate.json.one(json_path="$.studyList")
    def studyList(self) -> list[StudyDto]: ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.hasRelation")
    def hasRelation(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.name")
    def name(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...


class Hierarchy(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=SponsorExtDto)
    @cjen.operate.json.one(json_path="$.payload.sponsorExtDtoList")
    def sponsorExtDtoList(self) -> list[SponsorExtDto]: ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...



HOST = "200.200.101.113"
import json


def wash(func):
    def wrapper(*args, **kwargs):
        if args[1].request.host != HOST and not args[1].request.path.startswith("/api/"):
            return func(*args, **kwargs)

    return wrapper


def empty_string(content: str):
    return len(content.strip()) == 0


def disable_elements(*elements):
    for element in elements:
        element.Disable()


def enable_elements(*elements):
    for element in elements:
        element.Enable()


def read_parameter():
    with open('parameter.json', 'r') as f:
        return json.loads(f.read())


def write_parameter(**kwargs):
    with open('parameter.json', 'w') as f:
        f.write(json.dumps(kwargs))


if __name__ == '__main__':
    print(read_parameter().get("a"))



import cjen
from cjen import MetaJson


class RelObject(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.userId")
    def userId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.userGroupUserRelId")
    def userGroupUserRelId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.roleUserGroupUserRelId")
    def roleUserGroupUserRelId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.userGroupId")
    def userGroupId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.roleId")
    def roleId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...


class UserGroupRoleRel(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=RelObject)
    @cjen.operate.json.one(json_path="$.payload")
    def list(self) -> list[RelObject]: ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...



from cjen.exceptions import JsonPathNotFoundErr

from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.hierarchies.portal_find_study import PortalFindStudy


class PortalStudyNoStart(StandardStep):
    Name = "portal_study_no_start.py"
    System = "not_startup_study"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        if not self.service.context[PortalFindStudy.Id]:
            PortalFindStudy(self.service, self.scenario).run()

    def _execute(self):
        super()._execute()
        self.service.hierarchies_study_systems(path_variable=self.path_variable())

    def life_cycle(self):
        return self.scenario.life_cycle()

    def call_back(self, **kwargs):
        try:
            self.service.context[self.System] = [app.id() for app in kwargs.get("systems").no_rel_apps()]
        except JsonPathNotFoundErr as je:
            self.service.context[self.System] = None

    def path_variable(self):
        return dict(study_id=self.service.context[PortalFindStudy.Id], env_id=self.service.context[self.life_cycle()])



from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.role.portal_find_role import PortalFindRole


class PortalCreateRole(StandardStep):
    Name = "portal_create_role.py"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        PortalFindRole(self.service, self.scenario).run()

    def ignore(self): return self.service.context[PortalFindRole.Id] is not None

    def category(self): return self.scenario.role().split("/")[0]

    def code(self): return self.scenario.role()

    def subCode(self): return self.scenario.role().split("/")[1]

    def data(self):
        return dict(category=self.category(), code=self.code(), subCode=self.subCode(), description="Test", active=True)

    def _execute(self):
        super()._execute()
        self.service.role_api_create_role(data=self.data())




import json
import os.path


class Memcached:
    def __init__(self):
        self.data = dict()

    def set(self, key, value):
        self.data[key] = value
        with open(os.path.join(os.path.dirname(__file__), 'memcached.tmp'), 'w') as f:
            f.write(json.dumps(self.data))

    def get(self, key):
        if not os.path.exists(os.path.join(os.path.dirname(__file__), 'memcached.tmp')): return None
        with open(os.path.join(os.path.dirname(__file__), 'memcached.tmp'), 'r') as f:
            return json.loads(f.read()).get(key)


memcached = Memcached()



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.hierarchies.portal_find_sponsor import PortalFindSponsor
from eclinical.standard.steps.portal.user_groups.portal_find_usergroup import PortalFindUserGroup
from eclinical.standard.steps.portal.user_groups.portal_get_company_envs import PortalGetCompanyEnvs
from eclinical.standard.steps.portal.user_groups.portal_get_lifecycle_hierarchies import PortalGetLifeCycleHierarchies


class PortalSetLifeCycleHierarchies(StandardStep):
    Name = "portal_set_lifecycle_hierarchies.py"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def life_cycle(self):
        return self.scenario.life_cycle()

    def user_group(self):
        return self.scenario.user_group()

    def _pre_processor(self):
        PortalGetLifeCycleHierarchies(self.service, self.scenario).run()

    def path_variable(self):
        for life_cycle in self.service.context[PortalGetCompanyEnvs.Object]:
            for life_cycle_name, life_cycle_id in life_cycle.items():
                if life_cycle_name == self.life_cycle():
                    return {"life_cycle_id": life_cycle_id, "userGroup_id": self.service.context[PortalFindUserGroup.Id]}
        return {"life_cycle_id": None}

    def sponsor_name(self):
        return self.scenario.sponsor()

    def study_name(self):
        return self.scenario.study()

    def sites_name(self):
        return self.scenario.user_group_sites()

    def data(self):
        for sponsor in self.service.context["hierarchies"]:
            if sponsor.name == self.sponsor_name():
                for study in sponsor.studyList:
                    if study.name == self.study_name():
                        for site in study.siteList:
                            if site.name in self.sites_name():
                                site.selected(self.service.context[PortalFindUserGroup.Id])
                        study.selected(self.service.context[PortalFindUserGroup.Id])
                sponsor.selected(self.service.context[PortalFindUserGroup.Id])
        return [sponsor.to_dict() for sponsor in self.service.context["hierarchies"]]

    def _execute(self):
        super()._execute()
        self.service.usergroups_api_set_lifecycle_hierarchies(data=self.data(), path_variable=self.path_variable())



class Radio:
    def __init__(self, msg: str, select: tuple):
        self.select = select
        self.msg = msg

    def __repr__(self):
        string = f"{self.msg}["
        string += ",".join([f"{i+1}:{v}" for i, v in enumerate(self.select)])
        string += "]"
        return string

    def get(self, index):
        if index < 1 or index > len(self.select):
            raise Exception(f"{index} 非法输入")
        return self.select[index-1]



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user_groups.portal_find_usergroup import PortalFindUserGroup
from eclinical.standard.steps.portal.user_groups.portal_get_company_envs import PortalGetCompanyEnvs


class PortalCreateUserGroup(StandardStep):
    Name = "portal_create_user_group"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        PortalFindUserGroup(self.service, self.scenario).run()
        PortalGetCompanyEnvs(self.service, self.scenario).run()

    def ignore(self): return self.service.context[PortalFindUserGroup.Id] is not None

    def code(self): return self.scenario.user_group()

    def _execute(self):
        super()._execute()
        if self.service.context[PortalFindUserGroup.Id] is None:
            self.service.usergroups_create_user_group(data=self.data())

    def data(self):
        return {"code": self.code(), "active": True, "envIds": self.service.context[PortalGetCompanyEnvs.Id]}



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario


class PortalGetCompanyEnvs(StandardStep):
    Name = "portal_get_company_envs"
    Id = "envIds"
    Object = "envObjects"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def call_back(self, **kwargs):
        self.service.context["envIds"] = [company_env.id() for company_env in kwargs.get("company_envs").list()]
        self.service.context[self.Object] = [{f'{company_env.name()}': company_env.id()} for company_env in
                                              kwargs.get("company_envs").list()]

    def _execute(self):
        super()._execute()
        self.service.usergroups_api_company_envs()



import re

from swagger import Swagger


class ApiSpec:

    def __init__(self, resource):
        self.apis = resource.apis(self.system)

    @property
    def system(self):
        return ""

    @classmethod
    def name(cls, http_path: str, resource_clazz):
        resource = resource_clazz()
        for api_spec_clazz in ([DESIGN, EDC, PV, ADMIN, IWRS, eTMF,  CTMS]):
            api_spec = api_spec_clazz(resource)
            if api_spec.system == http_path.split("/")[0]:
                return api_spec.spec(http_path)
        raise  Exception("没有该系统")

    def spec(self, http_path):
        for api in self.apis:
            api_regex = re.sub(r'\{\w+[Id | id | ID]\}', '\\\d+', f'{self.system}{api}')
            api_regex = re.sub(r'\{env}', '\\\d+', api_regex)
            api_regex = re.sub(r'\{location\}', '\\\d+', api_regex)
            if re.match(api_regex, http_path):
                return f'{self.system}{api}'
        return http_path


class DESIGN(ApiSpec):
    @property
    def system(self): return "design"


class EDC(ApiSpec):
    @property
    def system(self): return "edc"


class IWRS(ApiSpec):
    @property
    def system(self): return "iwrs"


class PV(ApiSpec):
    @property
    def system(self): return "pv"


class ADMIN(ApiSpec):
    @property
    def system(self): return "admin"


class eTMF(ApiSpec):
    @property
    def system(self): return "etmf"


class CTMS(ApiSpec):
    @property
    def system(self): return "ctms"


if __name__ == '__main__':
    print(ApiSpec.name("design/study/12/switch", Swagger))



import os.path
from eclinical import Environment
from eclinical.standard.portal.portal_api import PortalApi

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user.portal_associated_usergroup import PortalAssociatedUserGroup
from eclinical.standard.steps.portal.user.portal_create_user import PortalCreateUser
from eclinical.standard.steps.portal.user.portal_find_user import PortalFindUser
from eclinical.standard.steps.portal.user.portal_find_user_role import PortalFindUserRole
from eclinical.standard.steps.portal.user.portal_study_title import PortalStudyTitle
from eclinical.standard.steps.portal.user.portal_user_access import PortalUserAccess
from eclinical.standard.steps.portal.user.portal_user_append_usergroup import PortalUserAppendUserGroup


class ScoCreateUser:
    def __init__(self, scenario_dir, environment: Environment):
        self.scenario = PortalScenario(scenario_dir)
        self.service = PortalApi(environment)
        self.scenario.append_step(PortalFindUser, self.service)
        self.scenario.append_step(PortalCreateUser, self.service)
        self.scenario.append_step(PortalFindUserRole, self.service)
        self.scenario.append_step(PortalAssociatedUserGroup, self.service)
        self.scenario.append_step(PortalUserAppendUserGroup, self.service)
        self.scenario.append_step(PortalUserAccess, self.service)
        self.scenario.append_step(PortalStudyTitle, self.service)

    def run(self): self.scenario.run()


if __name__ == '__main__':
    sco_dir = os.path.join("new_study")
    envir = Environment.loader("DEV_01")
    sco = ScoCreateUser(sco_dir, envir)
    sco.run()



import cjen

from eclinical import CTMSLoginService, Environment
from eclinical.standard.base.ok_response import OkResponse
from eclinical.standard.ctms.dto.country import Countries
from eclinical.standard.ctms.dto.menus import Menus
from eclinical.standard.ctms.dto.site_entities import SiteEntities
from eclinical.standard.ctms.dto.sites_of_study import SitesOfStudy
from eclinical.standard.ctms.dto.studies import Studies
from eclinical.standard.steps.ctms.ctms_all_menu import CtmsAllMenu
from eclinical.standard.steps.ctms.ctms_common_country import CtmsCommonCountry
from eclinical.standard.steps.ctms.ctms_find_site import CtmsFindSite
from eclinical.standard.steps.ctms.ctms_find_study import CtmsFindStudy
from eclinical.standard.steps.ctms.ctms_find_study_site import CtmsFindStudySite


class CtmsApi(CTMSLoginService):
    def __init__(self, environment: Environment = None):
        super().__init__(environment)

    @cjen.http.get_mapping(uri="ctms/user/location/0/menu", json_clazz=Menus)
    @cjen.step.call(stepName=CtmsAllMenu.Name, argName="menus")
    def common_menu(self, menus: Menus, resp=None, **kwargs):
        ...

    @cjen.http.get_mapping(uri="ctms/common/country", json_clazz=Countries)
    @cjen.step.call(stepName=CtmsCommonCountry.Name, argName="countries")
    def common_country(self, countries: Countries, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="ctms/site/information/list?pageNo={pageNo}&pageSize=25", json_clazz=SitesOfStudy)
    @cjen.step.call(stepName=CtmsFindStudySite.Name, argName="query")
    def study_management_site_list(self, path_variable, data, query: SitesOfStudy = None, resp=None, **kwargs):
        if not query.nextPage() == 0:
            self.study_management_site_list(path_variable=dict(pageNo=query.nextPage()), data=data)

    @cjen.http.post_mapping(uri="ctms/entity/site/list?pageNo={pageNo}&pageSize=25", json_clazz=SiteEntities)
    @cjen.step.call(stepName=CtmsFindSite.Name, argName="site_entities")
    def entity_management_site_list(self, path_variable, site_entities: SiteEntities, resp=None, **kwargs):
        if CtmsFindSite.NoFill in self.context[CtmsFindSite.Info].values():
            if not site_entities.nextPage() == 0:
                self.entity_management_site_list(path_variable=dict(pageNo=site_entities.nextPage()))

    @cjen.http.upload_mapping(uri="ctms/entity/site/info/save")
    def entity_management_add_site(self, data, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="ctms/study/basic/info", json_clazz=OkResponse)
    def study_management_submit_info(self, data, ok: OkResponse, resp=None, **kwargs):
        ...

    @cjen.http.upload_mapping(uri="ctms/site/basic/info", json_clazz=OkResponse)
    @cjen.operate.asserts.validation_meta(meta_name="ok", fields="procCode")
    def study_management_add_site(self, data, ok: OkResponse, resp=None, **kwargs):
        ...

    @cjen.http.post_mapping(uri="ctms/study/query?pageNo=1&pageSize=25", json_clazz=Studies)
    @cjen.step.call(stepName=CtmsFindStudy.Name, argName="studies")
    def study_management_get_study(self, studies: Studies = None, resp=None, **kwargs):
        ...


if __name__ == "__main__":
    ctms = CtmsApi(Environment.loader("DEV_03"))
    for site in [[100990, "D007"],
                 [101000, "D008"],
                 [101010, "D009"],
                 [101020, "D010"],
                 [101030, "D011"],
                 [101040, "D012"],
                 [101050, "D013"],
                 [101060, "D014"],
                 [101070, "D015"],
                 [101080, "D016"],
                 [101090, "D017"],
                 [101100, "D018"],
                 [101110, "D019"],
                 [101120, "D020"],
                 [101130, "D021"],
                 [101140, "D022"],
                 [101150, "D023"],
                 [101160, "D024"],
                 [101170, "D025"],
                 [101180, "D026"],
                 [102370, "D030"],
                 [102380, "D027"],
                 [102390, "D028"],
                 [102400, "D029"]]:
        ctms.study_management_add_site(data={"siteId": f'{site[0]}',
                                             "siteName": site[1],
                                             "siteCode": site[1],
                                             "timezoneId": '0',
                                             "languageId": '2',
                                             "expectedSubject": '100',
                                             "studyId": '818',
                                             "menuId": '5080', })



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user_groups.portal_find_usergroup import PortalFindUserGroup


class PortalFindUnAddedUser(StandardStep):
    Name = "portal_get_un_added_user.py"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        PortalFindUserGroup(self.service, self.scenario).run()

    def call_back(self, **kwargs):
        self.service.context["user_id"] = None
        for user in kwargs.get("group_users").list():
            if user.loginName() == self.user_name():
                self.service.context["user_id"] = user.id()

    def user_name(self): return self.scenario.user()

    def _execute(self):
        super()._execute()
        self.service.usergroups_api_get_un_added_user(path_variable=self.path_variable())

    def path_variable(self): return dict(userGroup_id=self.service.context["user_group_id"])



import warnings


def error(err_msg):
    def __wrapper__(func):
        def __inner__(self, *args, **kwargs):
            result = func(self, *args, **kwargs)
            if result[0] is None or len(str(result[0])) == 0: raise Exception(f"{self.file_path} {result[1]} {err_msg}")
            return result[0]
        return __inner__

    return __wrapper__


def warning(warn_msg):
    def __wrapper__(func):
        def __inner__(self, *args, **kwargs):
            result = func(self, *args, **kwargs)
            if result[0] is None or len(str(result[0])) == 0:
                warnings.warn(f"{self.file_path} {result[1]} {warn_msg}", Warning, stacklevel=4)
            return result[0]
        return __inner__

    return __wrapper__


def required(func):
    @error(err_msg=f"不能为空")
    def __inner__(self, *args, **kwargs):
        return func(self, *args, **kwargs)
    return __inner__


def optional(func):
    @warning(warn_msg=f"为空")
    def __inner__(self, *args, **kwargs):
        return func(self, *args, **kwargs)
    return __inner__



from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user.portal_find_user import PortalFindUser


class PortalAssociatedUserGroup(StandardStep):
    Name = "portal_associate_user_group.py"
    ExistsRels = "exists_user_rels"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def _pre_processor(self):
        PortalFindUser(self.service, self.scenario).run()

    def path_variable(self):
        return dict(user_id=self.service.context[PortalFindUser.Id])

    def _execute(self):
        super()._execute()
        self.service.user_api_user_group_role_rel(path_variable=self.path_variable())

    def call_back(self, **kwargs):
        rels = [dict(roleId=rel.roleId(), userGroupId=rel.userGroupId(), userId=rel.userId()) for rel in kwargs.get("rel").list()]
        self.service.context[PortalAssociatedUserGroup.ExistsRels] = rels



# -*- coding: utf-8 -*-
from urllib.parse import urlparse

import wx
import wx.xrc
from mitmproxy.tools.main import mitmweb

# from record.parameter import Parameter
# from record.record_service import RecordService
from eclinical.record.ext.common import disable_elements, enable_elements, empty_string
from eclinical.record.ext.memcached import memcached
from eclinical.record.ext.proxy import Proxy, win_proxy
from eclinical.record.mitm.mitm_process import MitmProcess
from eclinical.record.view.alert_dialog import AlertDialog
from eclinical.record.view.status import Status


# from test import mock_upload, mock_upload_error, mock_record_error

# TODO 保存在工程目录中
# TODO 配置读取Enviroment文件
class RecordUi(wx.Frame):

    def __init__(self, parent, envirs):
        self.status_list = [Status.INIT]
        self.proxy = Proxy()
        self.record_service = None
        self.envirs = envirs
        # height = len(envirs) * 35
        wx.Frame.__init__(self, parent, id=wx.ID_ANY, title="接口录制", pos=wx.DefaultPosition,
                          size=wx.Size(self.width(), self.height()), style=wx.DEFAULT_FRAME_STYLE | wx.TAB_TRAVERSAL)

        self.SetSizeHints(wx.Size(self.width(), self.height()), wx.Size(self.width(), self.height()))
        self.mitm_process = None
        fgSizer1 = wx.FlexGridSizer(4, 2, 0, 0)
        fgSizer1.SetFlexibleDirection(wx.BOTH)
        fgSizer1.SetNonFlexibleGrowMode(wx.FLEX_GROWMODE_SPECIFIED)

        self.m_staticText3 = wx.StaticText(self, wx.ID_ANY, u"服务器列表", wx.DefaultPosition, wx.DefaultSize, 0)
        self.m_staticText3.Wrap(-1)

        fgSizer1.Add(self.m_staticText3, 0, wx.ALL, 5)

        # fgSizer1.Add(self.m_staticText41, 0, wx.ALL, 5)

        m_listBox2Choices = [name for name in envirs.keys()]
        self.m_listBox2 = wx.ListBox(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, m_listBox2Choices, 0)

        fgSizer1.Add(self.m_listBox2, 0, wx.ALL, 5)

        self.m_staticText4 = wx.StaticText(self, wx.ID_ANY, u"Json存放路径", wx.DefaultPosition, wx.DefaultSize, 0)
        self.m_staticText4.Wrap(-1)

        fgSizer1.Add(self.m_staticText4, 0, wx.ALL, 5)

        self.test_name = wx.TextCtrl(self, wx.ID_ANY, "ecd", wx.DefaultPosition, wx.DefaultSize, 0)
        self.test_name.Disable()
        fgSizer1.Add(self.test_name, 0, wx.ALL, 5)

        self.start_button = wx.Button(self, wx.ID_ANY, u"启动", wx.DefaultPosition, wx.DefaultSize, 0)
        self.start_button.Enable()
        fgSizer1.Add(self.start_button, 0, wx.ALL, 5)

        self.stop_button = wx.Button(self, wx.ID_ANY, u"停止", wx.DefaultPosition, wx.DefaultSize, 0)
        self.stop_button.Disable()
        fgSizer1.Add(self.stop_button, 0, wx.ALL, 5)

        # self.upload_button = wx.Button(self, wx.ID_ANY, u"上传", wx.DefaultPosition, wx.DefaultSize, 0)
        # self.upload_button.Disable()
        #
        # fgSizer1.Add(self.upload_button, 0, wx.ALL, 5)
        # self.m_dirPicker1 = wx.DirPickerCtrl(self, wx.ID_ANY, wx.EmptyString, u"请选择文件夹", wx.DefaultPosition,
        #                                      wx.DefaultSize,
        #                                      wx.DIRP_DEFAULT_STYLE)
        # fgSizer1.Add(self.m_dirPicker1, 0, wx.ALL, 5)
        #
        self.SetSizer(fgSizer1)
        self.Layout()

        self.Centre(wx.BOTH)
        # Connect Events
        self.start_button.Bind(wx.EVT_LEFT_UP, self.evt_start_up)
        self.stop_button.Bind(wx.EVT_LEFT_UP, self.evt_shut_down)
        # self.m_dirPicker1.Bind(wx.EVT_DIRPICKER_CHANGED, self.evt_select_folder)

    def __del__(self):
        pass

    def height(self):
        if len(self.envirs) * 25 + 200 > 326: return 326
        return len(self.envirs) * 10 + 200

    def width(self):
        max_item = 0

        def chinese(ch):
            return '\u4e00' <= ch <= '\u9fff'

        for value in self.envirs.keys():
            item = 0
            for ch in value:
                if chinese(ch):
                    item += 18
                else:
                    item += 8
            if item > max_item:
                max_item = item
        if max_item + 112 > 256: return max_item + 112 + 30
        return 256

    def change_status(self, new_status, msg=None):
        self.__append(new_status)
        self.status_evt(self.__peek(), msg)
        self.__pops(Status.UPLOADING_FINISH, Status.PREPARE_UPLOADING)
        self.__pops(Status.RECORDING_FINISH, Status.PREPARE_RECORDING)
        print(self.status_list)

    def __suspend(self):
        while True:
            status = self.__peek()
            if status == Status.PREPARE_RECORDING: break
            if status == Status.PREPARE_UPLOADING: break
            self.__pop()

    def __pops(self, begin, end):
        if self.__peek() == begin:
            while True:
                status = self.__pop()
                if status == end: break

    def __append(self, status):
        if self.__peek() == status:
            pass
        elif status == Status.PREPARE_RECORDING or status == Status.PREPARE_UPLOADING:
            if self.status_list[len(self.status_list) - 1] == Status.INIT:
                self.status_list.append(status)
            else:
                self.status_list[len(self.status_list) - 1] = status
        else:
            self.status_list.append(status)

    def __pop(self):
        return self.status_list.pop()

    def __peek(self):
        return self.status_list[len(self.status_list) - 1]

    def evt_start_up(self, event):
        self.change_status(Status.PREPARE_RECORDING)
        self.change_status(Status.RECORDING)
        if self.validation_record():
            memcached.set("host", self.parameter_host())
            memcached.set("folder", self.parameter_test_name())
            self.service_record_start()

    def evt_shut_down(self, event):
        self.change_status(Status.RECORDING_FINISH)
        self.service_record_stop()

    # def evt_select_folder(self, event):
    #     self.change_status(Status.PREPARE_UPLOADING)
    #
    # def validation_upload(self):
    #     if empty_string(self.m_dirPicker1.GetPath()):
    #         self.change_status(Status.ERROR, u"测试的数据文件不正确")
    #         return False
    #     return True

    def validation_record(self):
        if self.m_listBox2.GetSelection() == -1:
            self.change_status(Status.ERROR, u"请选择服务器列表")
            return False

        if empty_string(self.test_name.GetValue()):
            self.change_status(Status.ERROR, u"请简诉录制内容")
            return False
        return True

    def ui_disable_running(self):
        disable_elements(self.start_button,
                         self.test_name, self.m_listBox2, self.stop_button)

    def ui_enable_all(self):
        enable_elements(self.start_button,
                        self.test_name, self.m_listBox2, self.stop_button)

    def status_evt(self, status, msg=None):

        if status == Status.RECORDING:
            self.ui_disable_running()
            enable_elements(self.stop_button)
        elif status == Status.PREPARE_RECORDING:
            self.ui_enable_all()
            # disable_elements(self.upload_button)
            disable_elements(self.stop_button)
        elif status == Status.ERROR:
            AlertDialog.open(self, msg)
            self.__suspend()
            self.change_status(self.__peek())
        elif status == Status.RECORDING_FINISH:
            self.ui_enable_all()
            disable_elements(self.stop_button)
        elif status == Status.PREPARE_UPLOADING:
            self.ui_enable_all()
        elif status == Status.UPLOADING:
            self.ui_disable_running()
        elif status == Status.UPLOADING_FINISH:
            self.ui_enable_all()
            disable_elements(self.stop_button)

    def parameter_host(self):
        return self.envirs.get(self.m_listBox2.GetStringSelection())

    def parameter_test_name(self):
        return self.test_name.GetValue()

    def service_record_start(self):
        win_proxy.open()
        self.get_mitm_process().set_host(urlparse(self.parameter_host()).hostname)

        self.get_mitm_process().start()
        # mitm_process.start()

    def get_mitm_process(self):
        if self.mitm_process is None:
            self.mitm_process = MitmProcess()
        return self.mitm_process

    def service_record_stop(self):
        win_proxy.close()
        self.get_mitm_process().stop()
        self.mitm_process = None



# from scaffold.command.edk import Edk
from eclinical.scaffold.inputs.component.radio import Radio


class EclinicalUri:

    @classmethod
    def input(cls, edk, msg: str = "请输入访问的eclinical地址",
              select=("113", "test-01", "dev-01", "dev-02", "dev-03", "us_dev", "us_demo4")):
        radio = Radio(msg, select)
        try:
            key = radio.get(int(input(f"{radio}\n")))
            return edk.eclinical_serve.get(key)
        except Exception as e:
            return cls.input(edk, "输入非法，请输入访问的eclinical地址")



import cjen
from cjen import MetaJson


class Menus(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.payload")
    def payload(self): ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...
    


import os.path

import mitmproxy
import wx.xrc
import yaml

from eclinical.record.ext.api.swagger import Swagger
from mitmproxy.script import concurrent

from eclinical.record.ext.eclinical_service import EclinicalService
from eclinical.record.ext.memcached import memcached
from eclinical.record.view.record_ui import RecordUi
import multiprocessing


class Player:

    def ui(self, envirs: dict):
        app = wx.App()
        multiprocessing.freeze_support()
        for system in ["design", "admin", "ctms", "etmf", "edc", "pv", "iwrs"]:
            Swagger().apis(system, action=Swagger.INIT)
        ui = RecordUi(None, envirs)
        ui.Show()
        app.MainLoop()

    # TODO 根据顺序生成yaml文件，文件名为序号
    # TODO url,method, 请求体，响应体
    # TODO 文件的上传
    def response(self, flow: mitmproxy.http.HTTPFlow):
        if memcached.get("host") in flow.request.pretty_url and flow.request.path.startswith("/api"):
            EclinicalService(flow).generate()
            # return flow
            # if "user/permissions" in flow.request.pretty_url:
            #     with open(flow.request.path.replace("/", "_"), 'w') as f:
            #         f.write(str(flow.response.content, 'utf-8'))


if __name__ == "__main__":
    with open(os.path.join(os.path.dirname(os.getcwd()), "environment.yaml"), "r", encoding="utf-8") as f:
        envirs = {name: env.get("uri") for name, env in yaml.load(f.read(), Loader=yaml.FullLoader).get("ENV").items()}
    # print(envirs)
    # envirs = {"1111111111111111111111111111111111111111111111111111111111111111111111111":"2222"}
    Player().ui(envirs)



class Text:
    def __init__(self, msg: str, default=None, default_key=None):
        self.msg = msg
        self.default = default
        self.default_key = default_key
        self.mapping = {"回车": ""}

    def __repr__(self):
        if self.default:
            return f"{self.msg}, 如使用[{self.default},请按{self.default_key}]"
        return self.msg

    def get(self, value):
        if not self.default:
            if value == "":
                self.msg = "输入不能为空,请再次输入"
                return self.get(input(f"{self}\n"))
            else:
                return value
        elif value == self.mapping.get(self.default_key, self.default_key):
            return self.default
        else:
            return value



from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.ctms_scenario import CtmsScenario
from eclinical.standard.steps.ctms.ctms_find_site import CtmsFindSite
from eclinical.standard.steps.ctms.ctms_find_study import CtmsFindStudy
from eclinical.standard.steps.ctms.ctms_find_study_site import CtmsFindStudySite


class CtmsStudySetSite(StandardStep):
    Name = "ctms_set_site.py"

    def __init__(self, service, scenario: CtmsScenario):
        super().__init__(service, scenario)

    def sites(self):
        return self.scenario.sites()

    def _pre_processor(self):
        CtmsFindSite(self.service, self.scenario).run()
        CtmsFindStudySite(self.service, self.scenario).run()

    def data(self):
        site_codes = []
        for site_code in self.sites():
            for exist_site in self.service.context[CtmsFindStudySite.Info]:
                if exist_site.siteCode() == site_code: continue
            site_codes.append(site_code)
        return [{"siteId": f"{self.service.context[CtmsFindSite.Info].get(site_code)}",
                 "siteName": site_code,
                 "siteCode": site_code,
                 "timezoneId": '0',
                 "languageId": '2',
                 "expectedSubject": '100',
                 "studyId": f'{self.service.context[CtmsFindStudy.Info].get("id")}',
                 "menuId": '5080', } for site_code in site_codes]

    def _execute(self):
        super()._execute()
        for site_data in self.data():
            self.service.study_management_add_site(data=site_data)



import cjen
from cjen import MetaJson


class SiteEntity(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.siteName")
    def siteName(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...


class SiteEntities(MetaJson):
    
    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.payload.nextPage")
    def nextPage(self): ...
    
    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=SiteEntity)
    @cjen.operate.json.one(json_path="$.payload.list")
    def list(self) -> list[SiteEntity]: ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...



import cjen

from eclinical import DesignLoginService, Environment
from eclinical.standard.design.dto.archive_file import ArchiveFiles
from eclinical.standard.design.dto.version_info import VersionInfo


class Versioning(DesignLoginService):
    def __init__(self, environment: Environment):
        super().__init__(environment)

    @cjen.http.get_mapping(uri="design/crf/version/list", json_clazz=VersionInfo)
    @cjen.operate.asserts.validation_meta(meta_name="version_info", fields="procCode")
    def version_list(self, version_info: VersionInfo = None, resp=None, **kwargs):
        from eclinical.standard.steps.design.versioning.design_get_last_version import DesignGetLastVersion
        self.step_definitions.call_back(DesignGetLastVersion.Name, version_info=version_info)

    @cjen.http.post_mapping(uri="design/crf/version/download")
    def download_file(self, data, resp=None, **kwargs):
        from eclinical.standard.steps.design.versioning.design_download_publish_sql import DesignDownloadPublishSql
        self.step_definitions.call_back(DesignDownloadPublishSql.Name, file_bytes=resp)

    @cjen.http.get_mapping(uri="design/crf/version/archive-file?currentVersionId={currentVersionId}",
                           json_clazz=ArchiveFiles)
    @cjen.operate.asserts.validation_meta(meta_name="archives", fields="procCode")
    def archive_files(self, path_variable, archives: ArchiveFiles = None, resp=None, **kwargs):
        from eclinical.standard.steps.design.versioning.design_fileids import DesignFileIDS
        self.step_definitions.call_back(DesignFileIDS.Name, archives=archives)



'''
Author: your name
Date: 2022-03-03 17:20:13
LastEditTime: 2022-03-03 17:44:13
LastEditors: your name
Description: 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
FilePath: \eClinical4.0_testing\eClinical\service\etmf_login_service.py
'''
import cjen

from eclinical.environment.environment import Environment
from eclinical.service._sponsor_login_service import _SponsorLoginService


class ETMFLoginService(_SponsorLoginService):
    @cjen.context.add(content=dict(system="eTMF"))
    def __init__(self, environment: Environment = None):
        super().__init__(environment)
        self.sponsor_auth()

    @cjen.http.post_mapping(uri="etmf/auth")
    @cjen.jwt(key="Authorization", json_path="$.payload.jwtAuthenticationResponse.token")
    def sponsor_auth(self, resp=None, **kwargs):
        ...



import argparse
import os
import shutil

from ruamel import yaml

from eclinical.scaffold.inputs.edk.account import Account
from eclinical.scaffold.inputs.edk.company import Company
from eclinical.scaffold.inputs.edk.db_host import DBHost
from eclinical.scaffold.inputs.edk.eclinical_uri import EclinicalUri
from eclinical.scaffold.inputs.edk.environment_name import EnvironmentName
from eclinical.scaffold.inputs.edk.life_cycle import LifeCycle
from eclinical.scaffold.inputs.edk.pass_word import PassWord
from eclinical.scaffold.inputs.edk.port import Port
from eclinical.scaffold.inputs.edk.schema import Schema
from eclinical.scaffold.inputs.edk.sponsor import Sponsor
from eclinical.scaffold.inputs.edk.study import Study

parser = argparse.ArgumentParser()
parser.add_argument("-init", action='store_true', help="初始化整个项目，包括生成environment.yaml，全局的conftest")
parser.add_argument("-new", action='store_true', help="新增加 environment 到 environment.yaml中 ")
parser.add_argument("-no_db", action='store_true', help="不需要配置数据库")
parser.add_argument("-no_test", action='store_true', help="不成生全局的conftest")
args = parser.parse_args()


class Edk:
    def __init__(self):
        self.environment_path = os.path.join(os.getcwd(), "environment.yaml")
        self.conftest_path = os.path.join(os.getcwd(), "conftest.py")

    def init(self):
        shutil.copyfile(os.path.join(os.path.dirname(__file__), "resource", "init_environment.yaml"),
                        self.environment_path)

    def generate_conftest(self):
        shutil.copyfile(os.path.join(os.path.dirname(__file__), "resource", "global_conftest.txt"), self.conftest_path)

    def add_environment(self, name: str, environment: dict):
        with open(self.environment_path, 'r', encoding="utf-8") as f:
            environments = yaml.load(f.read(), Loader=yaml.Loader)
            environments["ENV"][name] = environment
        with open(self.environment_path, 'w', encoding="utf-8") as f:
            yaml.dump(environments, f, Dumper=yaml.RoundTripDumper, allow_unicode=True)

    def environment_exist(self, name):
        with open(self.environment_path, 'r', encoding="utf-8") as f:
            environments = yaml.load(f.read(), Loader=yaml.Loader)
            return environments["ENV"].get(name) is not None

    def exist(self): return os.path.exists(self.environment_path)

    @property
    def eclinical_serve(self):
        return {"113": "http://200.200.101.113/api",
                "test-01": "http://52.82.79.202:81/api",
                "dev-01": "http://161.189.155.243:81/api",
                "dev-02": "http://161.189.5.162:81/api",
                "dev-03": "http://52.82.113.74:81/api",
                "us_dev": "https://ec.eclinical-dev.edetekapps.com/api",
                "us_demo4": "https://ec4.ec4demo.edetekapps.com/api"}


def edk_command():
    try:
        edk = Edk()
        if args.init:
            if edk.exist():
                raise Exception("已初始化，请勿重复初始化")
            edk.init()
            if not args.no_test:
                edk.generate_conftest()
        if args.new:
            if not edk.exist(): raise Exception("请先执行-init 初始化，再执行-new")
            data = dict()
            environment_name = EnvironmentName.input(edk)
            data["uri"] = EclinicalUri.input(edk)
            data["user"] = Account.input()
            data["password"] = PassWord.input("请输入用户名密码", "Admin@123", "回车")
            data["public_key"] = input("加密公钥\n")
            data["company"] = Company.input("请输入公司名", "非Cross账号", "回车")

            data["sponsor"] = Sponsor.input()
            data["study"] = Study.input("请输入实验名", "ctms,pv,etmf", "回车")
            data["life_cycle"] = LifeCycle.input()
            if not args.no_db:
                data["db"] = dict()

                data["db"]["host"] = DBHost.input()
                data["db"]["user"] = Account.input()
                data["db"]["pwd"] = PassWord.input("请输入数据库密码")

                data["db"]["database"] = Schema.input()
                data["db"]["port"] = Port.input("请输入访问端口号", 3306, "回车")
            edk.add_environment(environment_name, data)
    except Exception as e:
        print(e)


if __name__ == '__main__':
    edk_command()



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.portal.dto.hierarchies.node import Node
from eclinical.standard.portal.dto.hierarchies.sponsor import Sponsor

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.hierarchies.portal_find_sponsor import PortalFindSponsor
from eclinical.standard.steps.portal.user_groups.portal_find_usergroup import PortalFindUserGroup
from eclinical.standard.steps.portal.user_groups.portal_get_company_envs import PortalGetCompanyEnvs


class PortalGetLifeCycleHierarchies(StandardStep):
    Name = "portal_get_lifecycle_hierarchies.py"
    Tree = "hierarchies"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def _pre_processor(self):
        PortalGetCompanyEnvs(self.service, self.scenario).run()

    def life_cycle(self):
        return self.scenario.life_cycle()

    def user_group(self):
        return self.scenario.user_group()

    def path_variable(self):
        for life_cycle in self.service.context[PortalGetCompanyEnvs.Object]:
            for life_cycle_name, life_cycle_id in life_cycle.items():
                if life_cycle_name == self.life_cycle():
                    return {"life_cycle_id": life_cycle_id, "userGroup_id": self.service.context[PortalFindUserGroup.Id]}
        return {"life_cycle_id": None}

    def call_back(self, **kwargs):
        self.service.context[self.Tree] = Sponsor.trees(kwargs.get("hierarchies").sponsorExtDtoList(),
                                                        self.service.context[PortalFindUserGroup.Id])

    def _execute(self):
        super()._execute()
        self.service.usergroups_api_get_lifecycle_hierarchies(path_variable=self.path_variable())




from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user_groups.portal_find_un_added_user import PortalFindUnAddedUser


class PortalAddGroupUser(StandardStep):
    Name = "portal_add_group_user.py"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        PortalFindUnAddedUser(self.service, self.scenario).run()

    def ignore(self): return self.service.context["user_id"] is None

    def _execute(self):
        super()._execute()
        self.service.usergroups_api_add_group_user(path_variable=self.path_variable(), data=self.data())

    def path_variable(self): return dict(userGroup_id=self.service.context["user_group_id"])

    def data(self):
        return [dict(userGroupId=self.service.context["user_group_id"], userId=self.service.context["user_id"])]




from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.hierarchies.portal_find_study import PortalFindStudy
from eclinical.standard.steps.portal.hierarchies.portal_study_no_start import PortalStudyNoStart


class PortalStartupStudy(StandardStep):
    Name = "portal_startup_study"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        PortalFindStudy(self.service, self.scenario).run()
        PortalStudyNoStart(self.service, self.scenario).run()

    def ignore(self):
        return not self.service.context[PortalStudyNoStart.System]

    def _execute(self):
        super()._execute()
        self.service.hierarchies_study_startup(
            data=self.data(),
            path_variable=self.path_variable())

    def life_cycle(self): return self.scenario.life_cycle()

    def data(self): return self.service.context[PortalStudyNoStart.System]

    def path_variable(self):
        return dict(study_id=self.service.context[PortalFindStudy.Id],
                    env_id=self.service.context[self.life_cycle()])



from enum import Enum


class Status(Enum):
    INIT = 0
    PREPARE_RECORDING = 2
    RECORDING = 3
    RECORDING_FINISH = 1
    ERROR = 4
    PREPARE_UPLOADING = 5
    UPLOADING = 6
    UPLOADING_FINISH = 7



from eclinical.standard.portal.dto.hierarchies.node import Node
from eclinical.standard.portal.dto.hierarchies.study import Study
from eclinical.standard.portal.dto.hierarchy import SponsorExtDto


class Sponsor(Node):
    def __init__(self, sponsorExtDto: SponsorExtDto, parent, relationObjectId):
        super().__init__(sponsorExtDto, parent, relationObjectId)
        # self.relationObjectId = relationObjectId
        self.studyList = [Study(study_dto, self, relationObjectId) for study_dto in sponsorExtDto.studyList()]
        self.level = 1

    # Sponsor 维度选择
    # Sponsor下的study 和 site 全选
    def top_selected(self, relationObjectId):
        self.hasRelation = True
        self.relationObjectId = relationObjectId
        for study in self.studyList:
            study.top_selected()

    def to_dict(self):
        result = super(Sponsor, self).to_dict()
        result["studyList"] = [study.to_dict() for study in self.studyList]
        return result

    @classmethod
    def trees(cls, sponsorExtDtoList: list[SponsorExtDto], relationObjectId):
        return [Sponsor(sponsorExtDto, None, relationObjectId) for sponsorExtDto in sponsorExtDtoList]


from eclinical.scaffold.inputs.component.text import Text


class Study:
    @classmethod
    def input(cls, msg: str = "请输入实验名", default=None, default_key=None):
        text = Text(msg, default, default_key)
        user_input = text.get(input(f"{text}\n"))
        study = "" if user_input == default else user_input
        return study




from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.hierarchies.portal_find_sponsor import PortalFindSponsor


class PortalFindStudy(StandardStep):
    Name = "portal_find_study"
    Id = "study_id"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        PortalFindSponsor(self.service, self.scenario).run()

    def sponsor(self):
        return self.scenario.sponsor()

    def study(self):
        return self.scenario.study()

    def call_back(self, **kwargs):
        self.service.context[self.Id] = None
        for study in kwargs.get("query").studies():
            if study.name() == self.study():
                self.service.context[self.Id] = study.id()

    def _execute(self):
        super()._execute()
        self.service.hierarchies_get_study(
            data=self.data(),
            path_variable=self.path_variable())


    def data(self):
        return dict(sponsorId=self.service.context[PortalFindSponsor.Id])

    def path_variable(self):
        return dict(pageNo=1)



import cjen
from cjen import MetaJson


class _SponsorLoginData(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.payload[?(@.systemName=='{system}')].systemId")
    def systemId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.payload[?(@.systemName=='{system}')].onboardSponsorList[?(@.name=='{"
                                     "sponsor}')].sponsorId")
    def sponsor_id(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.payload[?(@.systemName=='{system}')].onboardSponsorList[?(@.name=='{"
                                     "sponsor}')].onboardEnvs[?("
                                     "@.name=='{env}')].id")
    def env_id(self): ...



from eclinical.standard.portal.dto.hierarchy import SponsorExtDto


class Node:
    def __init__(self, value, parent, relationObjectId):
        self.parent = parent
        self.hasRelation = value.hasRelation()
        self.relationObjectId = relationObjectId if self.hasRelation else None
        self.id = value.id()
        self.level = None
        self.name = value.name()
        self.parentId = parent.id if parent is not None else None

    def selected(self, relationObjectId):
        self.hasRelation = True
        self.relationObjectId = relationObjectId


    def to_dict(self):
        return dict(hasRelation=self.hasRelation,
                    id=self.id, level=self.level, name=self.name, parentId=self.parentId,
                    relationObjectId=self.relationObjectId)





import json
import os.path

import cjen
import re


class Swagger(cjen.BigTangerine):
    INIT = 1
    GET = 2

    def __init__(self):
        self.__setattr__("base_url", "http://52.82.79.202:81/")
        super().__init__()

    @cjen.http.get_mapping(uri="api/{system}/v2/api-docs")
    def doc(self, path_variable, resp=None, **kwargs):
        with open(os.path.join(os.path.curdir, f'swagger_{path_variable.get("system")}.json'), "w") as f:
            f.write(json.dumps(resp.get("paths")))

    @classmethod
    def with_param(cls, key):
        if re.findall(r'{([^}]+)}', key):
            return True
        return False

    def apis(self, system, action=GET):
        if not os.path.exists(os.path.join(os.path.curdir, f'swagger_{system}.json')):
            self.doc(path_variable=dict(system=system))
        if action == self.GET:
            with open(os.path.join(os.path.curdir, f'swagger_{system}.json'), "r") as f:
                try:
                    apis = json.loads(f.read()).keys()
                    apis = list(filter(self.with_param, apis))
                    return (key for key in apis)
                except Exception as e:
                    return []


if __name__ == '__main__':
    print(Swagger().apis("design"))



import os

from eclinical.environment.environment import Environment


class FunctionEnvironment(Environment):
    def __init__(self, *, envir, path):
        super().__init__(envir, os.path.join(path, "function_environment.yaml"))


import os
import argparse
from typing import IO
import re

OK_RESPONSE = """
import cjen
from cjen import MetaJson


class OKResponse(MetaJson):
    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...
"""


def clazz_name(name: str):
    return "".join([elem.capitalize() for elem in name.split("_")])


def login_service(name: str):
    login = dict(etmf="ETMFLoginService", pv="PVLoginService", ctms="CTMSLoginService", edc="EdcLoginService",
                 iwrs="IWRSLoginService", design="DesignLoginService", portal="PortalLoginService",
                 portaladmin="PortalAdministratorLoginService").get(name.lower())
    if login is None: raise Exception(f"不支持 {name} ")
    return login


class Service:
    def __init__(self, service_name, app):
        self.service_name = service_name
        self.app = app

    @property
    def name(self): return f"{self.service_name}_service"

    @property
    def application(self): return self.app

    def output_conftest_py(self, f: IO):
        f.write(f"from .service.{self.name} import {clazz_name(self.name)}\n\n\n")
        f.write("@pytest.fixture \n")
        f.write(f"def {self.name}(ef, st):\n")
        f.write(f"\treturn {clazz_name(self.name)}(Environment(envir=st, file_path=ef))\n\n\n")

    def output_init_py(self, f: IO):
        f.write(f"from .{self.name} import {clazz_name(self.name)}\n")

    def output_service_py(self, f: IO):
        f.write("""
import cjen
from eclinical import {application_service}, Environment


class {service_name}({application_service}):
    def __init__(self, environment: Environment):
        super().__init__(environment)
""".format(service_name=clazz_name(self.name),
           application_service=login_service(self.application)))

    @classmethod
    def factory(cls, services: str, application: str) -> list:
        if services and len(services) != 0:
            return [Service(service, application) for service in services.split(",")]
        return []


class InitPy:
    def __init__(self, services: list[Service]):
        self.services = services

    def output(self, f: IO):
        for service in self.services:
            service.output_init_py(f)


class ConfTest:
    def __init__(self, services: list[Service]):
        self.services = services

    def output(self, f: IO, action: str):
        if action == "w":
            f.write("import pytest\n")
            f.write("from eclinical import Environment\n")
        for service in self.services:
            service.output_conftest_py(f)


class ReadMe:

    def __init__(self):
        self.content = """
# 用例描述
描述该用例测试内容
# 实现
描述如何实现该用例
# 注意
执行用例中需要注意点
# 启动
启动的py文件
# 是否加入Jenkins 自动执行
"""

    def output(self, f: IO):
        f.write(self.content)


class Scaffold(object):

    def __init__(self, name: str, services: list[Service], append_action: bool = False):
        self.name = name
        self.services = services
        self.init_py = InitPy(self.services)
        self.conftest = ConfTest(self.services)
        self.read_me = ReadMe()

        self.append_action = append_action

    def generate_read_me(self):
        with open(os.path.join(self.name, "README.md"), "w", encoding="utf-8") as f:
            self.read_me.output(f)

    def generate_meta(self):
        os.mkdir(os.path.join(self.name, "meta"))
        with open(os.path.join(self.name, "meta", "ok_response.py"), "w") as f:
            f.write(OK_RESPONSE)

    def init_conftest(self):
        with open(os.path.join(self.name, "conftest.py"), "w") as f:
            self.conftest.output(f, "w")

    def append_conftest(self):
        with open(os.path.join(self.name, "conftest.py"), "a") as f:
            self.conftest.output(f, "a")

    def generate_test_case(self):
        os.mkdir(self.name)
        self.generate_meta()
        self.init_service()
        self.generate_read_me()
        self.init_conftest()

    def init_service(self):
        if not os.path.exists(os.path.join(self.name, "service")):
            os.mkdir(os.path.join(self.name, "service"))
        self.write_init(mode="w")
        for service in self.services:
            self.write_service(service, 'w')

    def write_service(self, service: Service, mode: str):
        with open(os.path.join(self.name, "service", f"{service.name}.py"), mode) as f:
            service.output_service_py(f)

    def write_init(self, mode: str):
        with open(os.path.join(self.name, "service", "__init__.py"), mode) as f:
            self.init_py.output(f)

    def append_service(self):
        if not os.path.exists(self.name): raise Exception(f" 无用例{self.name} 的目录")
        for service in self.services:
            if os.path.exists(os.path.join(self.name, "service", f"{service.name}.py")):
                action = input(f"{service.name}已存在,是否覆盖(y/n)?")
                if 'y' != action.lower() and 'n' != action.lower(): raise Exception("非法输入,请输入y/n")
                if 'n' == action: continue
            else:
                self.write_init(mode='a')
                self.append_conftest()
            self.write_service(service, mode='w')

    def execute(self):
        if self.append_action:
            self.append_service()
        else:
            self.generate_test_case()


class ScaffoldGM(object):
    def __init__(self, name: str, service: str, meta_type: str, meta_name: str, props: list[str]):
        self.name = name
        self.service = service
        self.meta_type = meta_type
        self.meta_name = meta_name
        self.props = props

    def write_json(self, meta_path: str):
        if self.meta_type.lower() == 'json':
            if not os.path.exists(os.path.join(meta_path, f"{self.service}_service")):
                os.mkdir(os.path.join(meta_path, f"{self.service}_service"))
            if os.path.exists(os.path.join(meta_path, f"{self.service}_service", f"{self.meta_name}.py")):
                action = input(f"{self.meta_name} 已存在,是否覆盖 y/n:\n")
                if action != 'y':
                    print("忽略")
                    return False
            with open(os.path.join(meta_path, f"{self.service}_service", f"{self.meta_name}.py"), encoding='utf-8',
                      mode="w") as f:
                f.write("import cjen\nfrom cjen import MetaJson\n\n\n")
                f.write(f"class {clazz_name(self.meta_name)}(MetaJson):\n")
                for prop in self.props:
                    f.write(f"\t@cjen.operate.common.value\n")
                    f.write(f"\t@cjen.operate.json.one(json_path=\"请填写jsonpath\")\n")
                    f.write(f"\tdef {prop}(self): ...\n\n")
            return True
        return False

    def write_mysql(self, meta_path: str):
        if self.meta_type.lower() == 'mysql':
            if not os.path.exists(os.path.join(meta_path, f"{self.service}_service")):
                os.mkdir(os.path.join(meta_path, f"{self.service}_service"))
            if os.path.exists(os.path.join(meta_path, f"{self.service}_service", f"{self.meta_name}.py")):
                action = input(f"{self.meta_name} 已存在,是否覆盖 y/n:\n")
                if action != 'y':
                    print("忽略")
                    return False
            with open(os.path.join(meta_path, f"{self.service}_service", f"{self.meta_name}.py"), mode="w") as f:
                f.write("import cjen\nfrom cjen import MetaMysql\n\n\n")
                f.write(f"class {clazz_name(self.meta_name)}(MetaMysql):\n")
                for prop in self.props:
                    f.write(f"\t@cjen.operate.common.value\n")
                    f.write(f"\tdef {prop}(self): ...\n\n")
            return True
        return False

    def execute(self):

        if os.path.basename(os.getcwd()) == self.name:
            # 在用例文件夹中
            meta_path = os.path.join(os.getcwd(), "meta")
            if self.meta_type.lower() == 'mysql':
                self.write_mysql(meta_path=meta_path)
            else:
                self.write_json(meta_path=meta_path)
        elif os.path.exists(os.path.join(os.getcwd(), self.name)):
            # 在用例文件夹的父文件夹中
            meta_path = os.path.join(os.getcwd(), self.name, "meta")
            if self.meta_type.lower() == 'mysql':
                self.write_mysql(meta_path=meta_path)
            else:
                self.write_json(meta_path=meta_path)
        else:
            raise Exception(f"can not find {self.name}")


class System:
    def __init__(self):
        self.apps = ["portaladmin", "portal", "ctms", "etmf", "design", "edc", "iwrs", "eConsent(暂时不支持)", "pv",
                     "IDP(暂不支持)"]

    def __repr__(self):
        string = "service为哪个系统["
        string += ",".join([f"{i}:{v}" for i, v in enumerate(self.apps)])
        string += "]"
        return string

    def get(self, index):
        if index == 7 or index == 9: raise Exception(self.apps[index])
        if index < 0 or index >= len(self.apps): raise Exception(f"{index} 非法输入")
        return self.apps[index]


class MetaTypes:
    def __init__(self):
        self.meta_types = ["mysql", "json"]

    def __repr__(self):
        string = "请选择meta对象类型["
        string += ",".join([f"{i}:{v}" for i, v in enumerate(self.meta_types)])
        string += "]"
        return string

    def get(self, index):
        if index < 0 or index >= len(self.meta_types):
            raise Exception(f"{index} 非法输入")
        return self.meta_types[index]


parser = argparse.ArgumentParser()
parser.add_argument("--t", type=str, required=True, help="用例的名字,必须指定")
parser.add_argument("-a", action='store_true', help="在已有的用例中,增加service,需添加该参数")
parser.add_argument("--app", type=str,
                    help=str(System()))
parser.add_argument("--ss", type=str,
                    help="创建的service文件, 命名规则'xx_service', 如果有多个的话,以','间隔\n "
                         "例子: --ss crf_page_service,dataset_page_service",
                    default="")

parser.add_argument("--gm", action="store_true", help="是否在meta文件夹中创建service文件夹， 必须通过--s 指定service")
parser.add_argument("--mn", help="meta名字, 配合 --gm命令, 例子: ok_response", default="")
parser.add_argument("--mt", help=f"meta类型,{str(MetaTypes())}, 配合 --gm命令, 例子: ok_response", default="")
parser.add_argument("--s", type=str,
                    help="添加meta 的 service文件, 配合 --gm命令",
                    default="")

parser.add_argument("--props", type=str,
                    help="设置meta 的属性, 如果有多个的话,以','间隔, 配合 --gm命令 \n"
                         "例子: --props id,name",
                    default="")

args = parser.parse_args()


def init_test_case(cmd_options):
    if not args.gm:
        services = cmd_options.ss if cmd_options.ss else input("请输入你想创建的service,命名方式'xx_yy',多个以','分割\n")
        system = System()
        application_index = input(f"{str(system)}\n")
        application = system.get(int(application_index))
        services = Service.factory(services=services, application=application)
        return Scaffold(name=cmd_options.t, services=services, append_action=cmd_options.a)
    return None


class ExistService:
    def __init__(self, name):
        if os.path.basename(os.getcwd()) == name:
            self.path = os.path.join(os.getcwd())
        elif os.path.exists(os.path.join(os.getcwd(), name)):
            # 在用例文件夹的父文件夹中
            self.path = os.path.join(os.getcwd(), name)
        self.cache = []
        self.scan()

    def scan(self):
        if os.path.exists(os.path.join(self.path, "service")):
            for root, dirs, files in os.walk(os.path.join(self.path, "service")):
                for file in files:
                    match_obj = re.match(r'(.*)_service.py$', file)
                    if match_obj: self.cache.append(match_obj.group(1))

    def __repr__(self):
        string = "请选择service序号,如无请手动输入service名["
        if self.cache:
            string += ",".join([f"{i + 1}:{v}" for i, v in enumerate(self.cache)])
        string += "]"
        return string

    def get(self, selected: str) -> str:
        match_obj = re.match(r"\d+", selected)
        if match_obj:
            index = int(match_obj.group())
            if index - 1 > len(self.cache) or index < 0: raise Exception(f"{index} 输入service 序号非法")
            return self.cache[index - 1]
        else:
            return selected


def init_meta(cmd_options):
    if args.gm:
        service_select = ExistService(cmd_options.t)
        if cmd_options.s:
            service = cmd_options.s
        else:
            service = service_select.get(input(f"{str(service_select)} 增加meta:\n"))
        meta_name = cmd_options.mn if cmd_options.mn else input("请输入meta文件名,命名方式'xx_yy':\n")

        if cmd_options.mt:
            meta_type = cmd_options.mt
        else:
            meta_index = input(f"{MetaTypes()}:\n")
            meta_type = MetaTypes().get(int(meta_index))
        props = cmd_options.props if cmd_options.props else input("请输入meta的属性(多个以,分割):\n")
        return ScaffoldGM(name=cmd_options.t, service=service, meta_type=meta_type, meta_name=meta_name,
                          props=props.split(","))


def run():
    try:
        scaffold = init_test_case(args) or init_meta(args)
        scaffold.execute()
    except Exception as e:
        print(e)

# if __name__ == '__main__':
    # print(str(ExistService("test")))
    # try:
    # scaffold = init_test_case(args) or init_meta(args)
    # scaffold.execute()
    # except Exception as e:
    #     print(e)



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.design.design_api import Versioning
from eclinical.standard.steps.design.versioning.design_fileids import DesignFileIDS


class DesignDownloadPublishSql(StandardStep):
    Name = "design_download_publish_sql.py"

    def __init__(self, service: Versioning, scenario: Scenario):
        super().__init__(service, scenario)

    def _pre_processor(self):
        DesignFileIDS(self.service, self.scenario).run()

    def _execute(self):
        super()._execute()
        self.service.download_file(data=self.data())

    def data(self):
        sql_file_id = self.service.context["archives"].db_spec().get("fileId") - 1
        return {"fileIds": [sql_file_id]}

    def call_back(self, **kwargs):
        with open("publish_sql.zip", 'wb') as f:
            f.write(kwargs.get("file_bytes"))



from eclinical.scaffold.inputs.component.text import Text


class MetaName:
    @classmethod
    def input(cls, msg: str = "请输入meta名 例如xx_yy", default=None, default_key=None):
        text = Text(msg, default, default_key)
        return text.get(input(f"{text}\n"))


import os

from eclinical.standard.steps.portal.hierarchies.portal_find_study import PortalFindStudy

from eclinical.standard.steps.portal.hierarchies.portal_find_sponsor import PortalFindSponsor

from eclinical import Environment

# 初始化整个公司账号的内容
from eclinical.standard.scenarios.sco_create_role import ScoCreateRole
from eclinical.standard.scenarios.sco_create_sponsor import ScoCreateSponsor
from eclinical.standard.scenarios.sco_create_study import ScoCreateStudy
from eclinical.standard.scenarios.sco_create_user import ScoCreateUser
from eclinical.standard.scenarios.sco_ctms_study import ScoCtmsStudy
from eclinical.standard.scenarios.sco_user_group import ScoUserGroup


class ScoInit:
    def __init__(self, scenario_dir, environment: Environment):
        self.scenario_dir = scenario_dir
        self.environment = environment
        self.sco_clazz = []

    def run(self):
        # 创建 Sponsor
        print("Portal 创建 Sponsor")
        sco = ScoCreateSponsor(self.scenario_dir, self.environment)
        sco.run()
        print(f'{sco.scenario.sponsor()} Sponsor ID: {sco.scenario.service.context[PortalFindSponsor.Id]}')
        # 创建 Study
        print("Portal 创建 Study")
        sco = ScoCreateStudy(self.scenario_dir, self.environment)
        sco.run()
        print(f'{sco.scenario.study()} Study ID: {sco.scenario.service.context[PortalFindStudy.Id]}')
        # 创建 Role
        print("Portal 创建 Role")
        ScoCreateRole(self.scenario_dir, self.environment).run()
        # 创建 UserGroup
        print("Portal 创建 User Group")
        ScoUserGroup(self.scenario_dir, self.environment).run()
        # 创建 User
        print("Portal 创建 User")
        ScoCreateUser(self.scenario_dir, self.environment).run()
        # 创建 CTMS SITE
        print("CTMS 创建Site")
        ScoCtmsStudy(self.scenario_dir, self.environment).run()
        # Portal 中分配Site
        print("Portal 为User Group 创建Site")
        ScoUserGroup(self.scenario_dir, self.environment).run()
        print("Finish")


if __name__ == '__main__':
    sco_dir = os.path.join("new_study")
    envir = Environment.loader("DEV_03")
    sco = ScoInit(sco_dir, envir)
    sco.run()



import cjen
from cjen import MetaJson


class CompanyMultipleEnv(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.defaultEnv")
    def defaultEnv(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.companyId")
    def companyId(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.alias")
    def alias(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.name")
    def name(self): ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...


class UserGroupInfo(MetaJson):

    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=CompanyMultipleEnv)
    @cjen.operate.json.one(json_path="$.payload.companyMultipleEnvs")
    def company_multiple_envs(self) -> list[CompanyMultipleEnv]: ...

    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...



import jsonpath
from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.design.design_api import Versioning


# 获取最后提交得版本信息
class DesignGetLastVersion(StandardStep):
    Name = "design_get_last_version"

    def __init__(self, service: Versioning, scenario: Scenario):
        super().__init__(service, scenario)


    def call_back(self, **kwargs):
        current = kwargs.get("version_info").current_version()
        if current.get("latest") is True and current.get("status") == 200:
            self.service.context["currentVersionId"] = current.get("id")
        else: raise Exception("The CRF has not Submit")

    def _execute(self):
        super()._execute()
        self.service.version_list()



import cjen
import eclinical

from eclinical.service._login_service import _LoginService
from eclinical.service.data._study_login_data import _StudyLoginData


class _StudyLoginService(_LoginService):

    def __init__(self, environment: eclinical.Environment):
        super().__init__(environment)
        self.auth(data=self.user_pwd())
        self.__cross_user() or self.__normal_user()
        self.user_onboard(
            data={k: v for k, v in self.context.content.items() if
                  k in ["applicationId", "sponsorId", "studyId", "envId", "workForCompanyId"]})

    def __cross_user(self):
        if self.context.get("company"):
            self.company()
            self.applications_cross_user(path_variable=dict(company_id=self.context.get("company_id")))
            self.context["workForCompanyId"] = self.context.get("company_id")
            return True
        return False

    def __normal_user(self):
        self.applications()

    @cjen.http.get_mapping(uri='admin/user/onboard/applications')
    @cjen.operate.json.factory(clazz=_StudyLoginData)
    def applications(self, resp=None, meta: _StudyLoginData = None, **kwargs) -> str:
        self.__update_login_info(meta)

    @cjen.http.get_mapping(uri='admin/user/onboard/applications?companyId={company_id}')
    @cjen.operate.json.factory(clazz=_StudyLoginData)
    def applications_cross_user(self, path_variable, resp=None, meta: _StudyLoginData = None, **kwargs):
        self.__update_login_info(meta)

    @cjen.http.post_mapping(uri="admin/user/onboard")
    @cjen.jwt(key="Authorization", json_path="$.payload.token")
    def user_onboard(self, data, resp=None, **kwargs):
        ...

    def __update_login_info(self, meta: _StudyLoginData):
        self.context["applicationId"] = meta.systemId()
        self.context["sponsorId"] = meta.sponsor_id()
        self.context["studyId"] = meta.study_id()
        self.context["envId"] = meta.env_id()


from eclinical.standard.portal.dto.hierarchies.node import Node
from eclinical.standard.portal.dto.hierarchy import SiteDto


class Site(Node):
    def __init__(self, value: SiteDto, parent:Node, relationObjectId):
        super().__init__(value, parent, relationObjectId)
        self.level = 3
        self.sponsorId = parent.parentId
        self.studyId = parent.id
        self.name = value.code()

    # 从Site 维度选择，对应的Study和Sponsor都应该选中
    def selected(self, relationObjectId):
        super(Site, self).selected(relationObjectId)
        self.parent.selected(relationObjectId)
    
    def to_dict(self):
        result = super(Site, self).to_dict()
        result["sponsorId"] = self.sponsorId
        result["studyId"] = self.studyId
        return result

    def __top_selected(self): self.hasRelation = True





from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.role.portal_find_role import PortalFindRole
from eclinical.standard.steps.portal.role.portal_get_permission_tree import PortalGetPermissionTree


class PortalSetPermissions(StandardStep):
    Name = "portal_set_permissions.py"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)


    def _pre_processor(self):
        PortalFindRole(self.service, self.scenario).run()
        PortalGetPermissionTree(self.service, self.scenario).run()

    def data(self):
        return [dict(permissionId=pid, roleId=self.service.context[PortalFindRole.Id])
                for pid in self.service.context[PortalGetPermissionTree.IdOfAllPermission]]

    def path_variable(self):
        return dict(role_id=self.service.context[PortalFindRole.Id])

    def _execute(self):
        super()._execute()
        self.service.role_api_set_permissions(path_variable=self.path_variable(), data=self.data())
